<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class Config {

    use Singleton;

    /**
     * Prepare a list of critical initial checks to run.
     * Available checks: 'php56', 'curl'
     *
     * @since 1.0.0
     * @access public
     *
     * @return array
     **/
    public function get_initial_checks() {

        return [
            'php56', // Plugin need PHP 5.6+.
            'curl' // Plugin need cURL extension.
        ];

    }

    /**
     * Prepare plugin features configurations by modifying the defaults.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    private function prepare_config() {

        /** Get default plugin config. */
        $config = Plugin::get_config();

        /** Merlin WP. */
        $config['merlin']['enabled'] = true;

        /** Show/Hide "Our Plugins" tab on "Add Plugins" page. */
        $config['our-plugins']['enabled'] = true;

        /** Update config. */
        Plugin::set_config( $config );

    }

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        /** Prepare custom plugin features configurations. */
        $this->prepare_config();

        /** Get default plugin settings. */
        $tabs = Plugin::get_tabs();

        /** Remove 'Delete plugin, settings and data' option from Uninstall tab. */
        unset( $tabs['uninstall']['fields']['delete_plugin']['options']['plugin+settings+data'] );

        /** Create General tab. */
        $tabs = $this->create_tab_general( $tabs );

        /** Create Copyright Dialog tab. */
        $tabs = $this->create_tab_copyright_dialog( $tabs );

        /** Modify Custom CSS tab. */
        $tabs = $this->modify_tab_custom_css( $tabs );

        /** Allow reset tab settings to default. */
        $tabs[ 'general' ]['reset_settings'] = true;
        $tabs[ 'assignments' ]['reset_settings'] = true;

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->refresh_options();

    }

    /**
     * Create General tab.
     *
     * @param array $tabs - List of tabs with all settings and fields.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array - List of tabs with all settings and fields.
     **/
    private function create_tab_general( $tabs ) {

        /** Shorthand for tab fields. */
        $fields = &$tabs[ 'general' ][ 'fields' ];

        /** Select All. */
        $fields[ 'select_all' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Select All:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Text from Being Copied by Select All HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+A, ⌘+A.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Copy. */
        $fields[ 'copy' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Copy:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Text from Being Copied by Copy HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+C, ⌘+C.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Cut. */
        $fields[ 'cut' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Cut:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Text from Being Copied by Cut HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+X, ⌘+X.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Paste. */
        $fields[ 'paste' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Paste:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable Paste HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+V, ⌘+V.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Save. */
        $fields[ 'save' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Save:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Text from Being Saved by Save HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+S, ⌘+S.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable View Source. */
        $fields[ 'view_source' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable View Source:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable to View Source Code of Page by HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+U, ⌘+U.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Print Page. */
        $fields[ 'print_page' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Print Page:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Page from Being Printed by HotKeys.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+P, ⌘+P.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Developer Tools. */
        $fields[ 'developer_tools' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Developer Tools:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable to View Source Code of Page by Developer Tools.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable HotKeys: Ctrl+Shift+I, ⌘+⌥+I.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Safari Reader. */
        $fields[ 'safari_reader' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Safari Reader:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Text and Images from being copied in the Safari Reader mode.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable Reader mode in Safari: ⌘+Shift+R.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Right Click. */
        $fields[ 'right_click' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Right Click:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Protect Your Content from Being Copied by Context Menu.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable Mouse Right Click.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Text Selection. */
        $fields[ 'text_selection' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Text Selection:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable Text Selection.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable Text Highlight by Mouse.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable Image Dragging. */
        $fields[ 'image_drag' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable Image Dragging:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable Image Dragging.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable Image Dragging by Mouse.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Disable RSS Feeds. */
        $fields[ 'rss_feed' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable RSS Feeds:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable RSS Feeds.', 't42-content-protector' ),
            'description'      => esc_html__( 'Disable RSS Feeds for Your Site.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** Divider. */
        $fields[ 'divider_js' ] = [ 'type' => 'divider' ];

        /** JavaScript Required. */
        $fields[ 'js_required' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'JavaScript Required:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'JavaScript Required.', 't42-content-protector' ),
            'description'      => esc_html__( 'Protect Content if JavaScript is Disabled.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'on',
        ];

        /** JavaScript Required Message. */
        $fields[ 'js_message' ] = [
            'type'             => 'editor',
            'label'            => esc_html__( 'JavaScript Required Message', 't42-content-protector' ),
            'show_label'       => true,
            'description'      => '',
            'show_description' => false,
            'default'          => '<h3>' . esc_html__( 'Please Enable JavaScript in your Browser to Visit this Site.', 't42-content-protector' ) . '</h3>',
            'attr'             => [
                'textarea_rows' => '3',
            ]
        ];

        /** Divider. */
        $fields[ 'divider_view_source' ] = [ 'type' => 'divider' ];

        /** View Source Note. */
        $fields['view_source_note'] = [
            'type'              => Controls::HTML,
            'content'           => $this->get_view_source_note(),
            'attr'              => [
                'class'     => 't42-content-protector-general-note',
            ]
        ];

        /** Disable view-source:{url}. */
        $fields[ 'view_source_scheme' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Disable view-source:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Disable view-source:{url}.', 't42-content-protector' ),
            'description'      => esc_html__( 'Protect content from being accessed via "view-source:{url}". 
                                  Please note that, for direct visitors on their first access, there will be two 
                                  initial page reloads. This option also blocks all clients 
                                  without JavaScript.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'off',
        ];

        return $tabs;

    }

    /**
     * Get content for general_note field.
     *
     * @since 2.0.0
     * @access private
     *
     * @return string
     **/
    private function get_view_source_note() {

        ob_start();
        ?>
        <div class="t42-alert t42-alert-primary" t42-alert="">
            <p>
                <?php esc_html_e( 'This option is experimental and may not work properly on some servers. 
                Please test it after enabling to make sure it works as expected. Open your site in incognito mode 
                and make sure that the content is protected. In case your host does not support this type of protection, 
                you will see an infinite page reload. In this case, disable this option immediately.', 't42-junction' ); ?>
            </p>
        </div>
        <?php

        return ob_get_clean();

    }

    /**
     * Create Copyright Dialog tab.
     *
     * @param array $tabs - List of tabs with all settings and fields.
     *
     * @since 1.1.0
     * @access private
     *
     * @return array - List of tabs with all settings and fields.
     **/
    private function create_tab_copyright_dialog( $tabs ) {

        /** Add Copyright Dialog tab after General. */
        $offset = 1; // Position for new tab.
        $tabs = array_slice( $tabs, 0, $offset, true ) +
                ['copyright_dialog' => [
                    'enabled'        => true,
                    'class'          => TabGeneral::class,
                    'label'          => esc_html__( 'Copyright Dialog', 't42-content-protector' ),
                    'title'          => esc_html__( 'Copyright Dialog', 't42-content-protector' ),
                    'show_title'     => true,
                    'reset_settings' => true,
                    'icon'           => 'fi-message-square',
                    'fields'         => []
                ] ] +
                array_slice( $tabs, $offset, NULL, true );


        /** Shorthand for tab fields. */
        $fields = &$tabs[ 'copyright_dialog' ][ 'fields' ];

        /** Enable Copyright Dialog. */
        $fields[ 'copyright_dialog' ] = [
            'type'             => 'switcher',
            'label'            => esc_html__( 'Copyright Dialog:', 't42-content-protector' ),
            'show_label'       => true,
            'placeholder'      => esc_html__( 'Show Copyright Dialog.', 't42-content-protector' ),
            'description'      => esc_html__( 'Enable Copyright warning when trying to copy.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => 'off',
        ];

        /** Copyright Warning Message. */
        $fields[ 'copyright_message' ] = [
            'type'             => 'editor',
            'label'            => esc_html__( 'Copyright Warning Message', 't42-content-protector' ),
            'show_label'       => true,
            'description'      => esc_html__( 'You can use special placeholders: ', 't42-content-protector' ) .
                                  '<em>{siteName}</em>,&nbsp; <em>{siteDomain}</em>,&nbsp; <em>{currentDate}</em>.' .
                                  esc_html__( ' They will be replaced with their values accordingly.', 't42-content-protector' ),
            'show_description' => true,
            'default'          => $this->get_copyright_message(),
            'attr'             => [
                'textarea_rows' => '15',
            ]
        ];

        /** Overlay color. */
        $fields[ 'overlay_color' ] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Overlay Color', 't42-content-protector' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select Color for Dialog Overlay.', 't42-content-protector' ),
            'show_description'  => true,
            'default'           => 'rgba(0,0,0,0.6)',
            'gradient_mode'     => true,
            'gradient_switcher' => 'off',
            'gradient_default'  => 'linear-gradient(0deg, rgba(46, 0, 1, 0.7) 0%,rgba(221, 32, 36, 0.63) 100%)',
            'attr'              => []
        ];

        /** Dialog color. */
        $fields[ 'dialog_color' ] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Dialog Color', 't42-content-protector' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select Color for Dialog Background.', 't42-content-protector' ),
            'show_description'  => true,
            'default'           => '#fff',
            'gradient_mode'     => true,
            'gradient_switcher' => 'off',
            'gradient_default'  => 'linear-gradient(0deg, rgba(46, 0, 1, 0.7) 0%,rgba(221, 32, 36, 0.63) 100%)',
            'attr'              => []
        ];

        return $tabs;
    }

    /**
     * Modify Custom CSS tab.
     *
     * @param array $tabs - List of tabs with all settings and fields.
     *
     * @since 1.1.0
     * @access private
     *
     * @return array - List of tabs with all settings and fields.
     **/
    private function modify_tab_custom_css( $tabs ) {

        /** Shorthand. */
        $fields = &$tabs[ 'custom_css' ][ 'fields' ];

        /** CSS Description. */
        $fields[ 'custom_css' ]['description'] =
            esc_html__( 'Add your custom CSS here. ', 't42-content-protector' ) .
            esc_html__( 'Please read first the ', 't42-content-protector' ) .
            '<a href="https://42theme.com/help-center/wordpress/t42-content-protector/content-protector-css-customisations/" target="_blank">' .
            esc_html__( 'CSS Customizations', 't42-content-protector' ) .
            '</a>' .
            esc_html__( ' section before you do any styling. ', 't42-content-protector' ) .
            esc_html__( 'It\'s very important to understand the class names and their meaning to not mess anything up.', 't42-content-protector' );

        return $tabs;

    }

    /**
     * Return HTML content for Copyright Message Dialog.
     *
     * @static
     * @since 1.0.8
     * @access private
     *
     * @return string
     **/
    private function get_copyright_message() {

        $html = '<h4>' . esc_html__( 'Copyright Notice.', 't42-content-protector' ) . '</h4>';
        $html .= '<p>' . esc_html__( 'All Rights Reserved.', 't42-content-protector' ) . '</p>';
        $html .= '<p>' . esc_html__( 'All material appearing on the {siteName} website ("content") is protected by copyright under U.S. Copyright laws and is the property of {siteName}. You may not copy, reproduce, distribute, publish, display, perform, modify, create derivative works, transmit, or in any way exploit any such content, nor may you distribute any part of this content over any network, including a local area network, sell or offer it for sale, or use such content to construct any kind of database.', 't42-content-protector' ) . '</p>';

        return $html;

    }

} // End class Config.
