<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Load JavaScripts for Frontend Area.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class FrontScripts extends Assets {

    use Singleton;

    /**
     * Sets up a new FrontScripts instance.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    private function __construct() {

        /** JavaScrips for frontend. */
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );

    }

    /**
     * Add JavaScrips for frontend area.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function enqueue_scripts() {

        /** Checks if plugin should work on this page. */
        if ( ! TabAssignments::get_instance()->display() ) { return; }

        /** JavaScript is not allowed in AMP. */
        if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) { return; }

        /** Disable content protector for Elementor and DIVI editor in the backend. */
        if ( Sense::get_instance()->is_page_builder() ) { return; }

        /** For development and debug. */
        $deps = [];
        if  ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) {

            /** Enqueue hotkeys.js script separately. */
            $this->enqueue_script( 'hotkeys' );
            $deps[] = $this->get_handle( 'hotkeys' );

            /**
             * Micromodal.js is a tiny, dependency-free javascript library for creating accessible modal dialogs.
             *
             * @see https://micromodal.vercel.app/
             * @see https://github.com/Ghosh/micromodal
             **/
            $this->enqueue_script( 'micromodal' );
            $deps[] = $this->get_handle( 'micromodal' );

            /**
             * Enqueue devtools-detector.js script separately.
             *
             * @see https://github.com/AEPKILL/devtools-detector
             **/
            $this->enqueue_script( 'devtools-detector' );
            $deps[] = $this->get_handle( 'devtools-detector' );

        }

        /** Enqueue Content protector script. */
        $this->enqueue_script( 'protector', false, $deps );

        /** Shorthand for plugin settings. */
        $opt = Settings::get_instance()->options;

        /** Prepare values to pass to JS. */
        $to_js = [
            'disableSelectAll'     => 'on' === $opt['select_all'],
            'disableCopy'          => 'on' === $opt['copy'],
            'disableCut'           => 'on' === $opt['cut'],
            'disablePaste'         => 'on' === $opt['paste'],
            'disableSave'          => 'on' === $opt['save'],
            'disableViewSource'    => 'on' === $opt['view_source'],
            'disablePrintPage'     => 'on' === $opt['print_page'],
            'disableDeveloperTool' => 'on' === $opt['developer_tools'],
            'disableReaderMode'    => 'on' === $opt['safari_reader'],
            'disableRightClick'    => 'on' === $opt['right_click'],
            'disableTextSelection' => 'on' === $opt['text_selection'],
            'disableImageDragging' => 'on' === $opt['image_drag'],
            'copyrightDialog'      => 'on' === $opt['copyright_dialog'],
        ];
        wp_localize_script( $this->get_handle( 'protector' ), 't42ContentProtectorOptions', $to_js );

    }

} // End class FrontScripts.
