<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Load CSS for Frontend Area.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class FrontStyles extends Assets {

    use Singleton;

    /**
     * Sets up a new FrontStyles instance.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    private function __construct() {

        /** CSS for frontend. */
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_styles' ] );

    }

    /**
     * Add CSS for frontend area.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function enqueue_styles() {

        /** Checks if plugin should work on this page. */
        if ( ! TabAssignments::get_instance()->display() ) { return; }

        /** Disable content protector for Elementor and DIVI editor in the backend. */
        if ( Sense::get_instance()->is_page_builder() ) { return; }

        /** Enqueue Content protector style. */
        $this->enqueue_style( 'protector' );

        /** Add inline styles. */
        wp_add_inline_style(
            $this->get_handle( 'protector' ),
            $this->get_inline_styles()
        );

    }

    /**
     * Return inline CSS custom scrollbar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string - CSS styles.
     **/
    private function get_inline_styles() {

        /** Get Overlay Color. */
        $overlay_color = $this->get_gradient_color( 'overlay_color' );

        /** Get Dialog Color. */
        $dialog_color = $this->get_gradient_color( 'dialog_color' );

        /** Resulting CSS. */
        $css = '';

        /** Dialog overlay color. */
        // language=CSS
        $css .= "	
            #t42-content-protector-copyright-dialog > div {
                background: $overlay_color;
            }
        ";

        /** Dialog color. */
        // language=CSS
        $css .= "
            #t42-content-protector-copyright-dialog > div > div {
                background: $dialog_color;
            }
        ";

        /** Add user's custom CSS from plugin settings. */
        $css .= Settings::get_instance()->options['custom_css'];

        return $css;

    }

    /**
     * Return valid CSS color or gradient value for field.
     *
     * @since 1.0.0
     * @access private
     *
     * @param string $key - Color Field key.
     *
     * @return string - CSS color or gradient value.
     **/
    private function get_gradient_color( $key ) {

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        /** Get color or gradient. */
        $color = $options[$key];
        if ( 'on' === $options[ $key . '_switcher' ] ) {
            $color = $options[ $key . '_gradient' ];
        }

        return $color;

    }

} // End class FrontStyles.
