<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * SINGLETON: The class contains the main plugin logic.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class Sense {

    use Singleton;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define public hooks. */
        $this->public_hooks();

    }

    /**
     * Register all hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Allow Google Page Speed Insights freely access to content. */
        if ( $this->is_google_page_speed_insights() ) { return; }

        /** Load CSS for Frontend Area. */
        /** @noinspection UnusedFunctionResultInspection */
        FrontStyles::get_instance();

        /** Load JavaScripts for Frontend Area. */
        /** @noinspection UnusedFunctionResultInspection */
        FrontScripts::get_instance();

        /** JavaScript Required protection. */
        add_action( 'wp_footer', [ $this, 'javascript_required' ] );

        /** Copyright Dialog. */
        add_action( 'wp_footer', [ $this, 'copyright_dialog' ] );

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        /** Work if "Disable view-source:{url}" Scheme is enabled. */
        if ( 'on' === $options[ 'view_source_scheme' ] ) {

            /** Start buffering to get a full HTML document. */
            add_action( 'template_redirect', [ $this, 'buffer_start' ], 0 );

            /** End buffering to get a full HTML document. */
            add_action( 'plugins_loaded', [ $this, 'buffer_end' ], PHP_INT_MAX );

        }

        /** Disable RSS feed. */
        if ( 'on' === $options[ 'rss_feed' ] ) {
            add_action( 'do_feed', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_rdf', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_rss', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_rss2', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_atom', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_rss2_comments', [ $this, 'disable_rss_feed' ], 1 );
            add_action( 'do_feed_atom_comments', [ $this, 'disable_rss_feed' ], 1 );
        }

    }

    /**
     * Disable RSS feed.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function disable_rss_feed() {
        wp_die( __( 'No feed available, please visit the <a href="'. esc_url( home_url( '/' ) ) .'">homepage</a>!' ) );
    }

    /**
     * Start buffering to get a full HTML document.
     *
     * @since 2.0.0
     * @access public
     *
     * return void
     **/
    public function buffer_start() {
        ob_start( [$this, 'process'] );
    }

    /**
     * End buffering to get a full HTML document.
     *
     * @since 2.0.0
     * @access public
     *
     * @return void
     **/
    public function buffer_end() {

        if ( ob_get_length() ) {
            ob_end_flush();
        }

    }

    /**
     * Performs all manipulations with the links.
     *
     * @param string $buffer - Full HTML of page.
     *
     * @since  2.0.0
     * @access public
     * @return string|string[]
     */
    public function process( $buffer ) {

        /** JavaScript is not allowed in AMP. */
        if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) { return $buffer; }

        /** Checks if plugin should work on this page. */
        if ( ! TabAssignments::get_instance()->display() ) { return $buffer; }

        /** Show normal content if the referrer is set. */
        $headers = getallheaders();
        if ( isset( $headers['Referer'] ) ) { return $buffer; }

        /** @noinspection JSCheckFunctionSignatures */
        $js = <<<JS
with(new Proxy({},{has(ﾠﾠﾠ,ᅠﾠ){let ﾠﾠ,ᅠ='';for(let ﾠ=0;ﾠ<ᅠﾠ.length;ﾠ+=8){ﾠﾠ=parseInt(ᅠﾠ.substr(ﾠ,8).replace(/ﾠ/g,0).replace(/ᅠ/g,1),2).toString(16);ᅠ+='%'+((ﾠﾠ.length%2==0)?ﾠﾠ:'0'+ﾠﾠ)}ᅠ=decodeURIComponent(ᅠ);console.log(ᅠ);eval(ᅠ);return ﾠ}}))
ﾠᅠᅠᅠﾠᅠᅠᅠﾠᅠᅠﾠᅠﾠﾠᅠﾠᅠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠﾠᅠﾠﾠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠᅠﾠᅠᅠᅠﾠﾠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠᅠᅠﾠﾠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠﾠﾠﾠᅠᅠﾠᅠᅠﾠﾠﾠﾠᅠﾠᅠᅠᅠﾠᅠﾠﾠﾠᅠᅠﾠᅠﾠﾠᅠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠﾠᅠᅠᅠﾠﾠﾠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠᅠﾠﾠﾠﾠᅠᅠᅠﾠﾠᅠﾠﾠᅠᅠﾠﾠᅠﾠᅠﾠᅠᅠﾠﾠᅠᅠﾠﾠﾠᅠﾠﾠﾠﾠﾠﾠﾠᅠᅠᅠᅠﾠᅠﾠﾠᅠﾠﾠﾠﾠﾠﾠᅠᅠᅠﾠᅠᅠᅠﾠᅠᅠﾠᅠﾠﾠᅠﾠᅠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠﾠᅠﾠﾠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠᅠﾠᅠᅠᅠﾠﾠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠᅠᅠﾠﾠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠﾠﾠﾠᅠᅠﾠᅠᅠﾠﾠﾠﾠᅠﾠᅠᅠᅠﾠᅠﾠﾠﾠᅠᅠﾠᅠﾠﾠᅠﾠᅠᅠﾠᅠᅠᅠᅠﾠᅠᅠﾠᅠᅠᅠﾠﾠﾠᅠﾠᅠᅠᅠﾠﾠᅠᅠﾠᅠﾠﾠﾠﾠᅠᅠᅠﾠﾠᅠﾠﾠᅠᅠﾠﾠᅠﾠᅠﾠᅠᅠﾠﾠᅠᅠﾠﾠﾠᅠᅠᅠﾠᅠᅠ
JS;

        /** Return empty body if "Disable view-source:{url}" Scheme is enabled. */
        return preg_replace(
                '/<body[^>]*>.*<\/body>/is',
            '<body>
<script>

' . $js . '
</script>
</body>',
                $buffer
        );

    }

    /**
     * Detect Google Page Speed Insights by User Agent.
     *
     * @since 1.0.7
     * @access public
     *
     * @return boolean TRUE for Google Page Speed Insights, FALSE otherwise.
     **/
    public function is_google_page_speed_insights() {

        /** Can't detect without a user agent. */
        if ( ! isset( $_SERVER['HTTP_USER_AGENT'] ) ) { return false; }

        /** Get the user agent. */
        $agent = $_SERVER["HTTP_USER_AGENT"];

        /** Substrings to detect Google Page Speed Insights. */
        $fingerprints = [
            'Google Page Speed Insights',
            'Chrome-Lighthouse',
            'Googlebot',
            'GoogleOther',
            'Google-Extended',
        ];

        /** Search for Google Page Speed Insights fingerprint. */
        foreach ( $fingerprints as $fingerprint ) {

            /** This is Google Page Speed Insights. */
            if ( strpos( $agent, $fingerprint ) !== false ) { return true; }

        }

        /** False for all other cases. */
        return false;

    }

    /**
     * Protect Content if JavaScript is Disabled.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function javascript_required() {

        /** JavaScript is not allowed in AMP. */
        if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) { return; }

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        /** Work if JavaScript Required. */
        if ( 'on' !== $options['js_required'] ) { return; }

        ob_start();
        ?>
        <noscript>
            <div id='t42-content-protector-js-disabled'>
                <div><?php echo wp_kses_post( $options['js_message'] ); ?></div>
            </div>
            <style>
                #t42-content-protector-js-disabled {
                    position: fixed;
                    top: 0;
                    left: 0;
                    height: 100%;
                    width: 100%;
                    z-index: 999999;
                    text-align: center;
                    background-color: #FFFFFF;
                    color: #000000;
                    font-size: 40px;
                    display: flex;
                    align-items: center;
                    justify-content: center;
                }
            </style>
        </noscript>
        <?php
        echo ob_get_clean();

    }

    /**
     * Render Copyright Dialog.
     *
     * @since 1.0.8
     * @access public
     *
     * @return void
     **/
    public function copyright_dialog() {

        /** Checks if plugin should work on this page. */
        if ( ! TabAssignments::get_instance()->display() ) { return; }

        /** JavaScript is not allowed in AMP. */
        if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) { return; }

        /** Disable content protector for Elementor and DIVI editor in the backend. */
        if ( self::get_instance()->is_page_builder() ) { return; }

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        /** Work only if Copyright Dialog enabled. */
        if ( 'on' !== $options['copyright_dialog'] ) { return; }

        /** Prepare Copyright Message. */
        $copyright_message = $this->apply_replacements();

        ?>
        <!-- Start Content Protector WordPress Plugin -->
        <div id="t42-content-protector-copyright-dialog" aria-hidden="true">
            <div tabindex="-1" data-micromodal-close="">
                <div role="dialog" aria-modal="true" aria-labelledby="t42-content-protector-copyright-dialog-title" >
                    <header>
                        <button aria-label="<?php esc_html_e( 'Close Modal', 't42-content-protector' ); ?>" data-micromodal-close=""></button>
                    </header>
                    <div id="t42-content-protector-copyright-dialog-content">
                        <?php echo wp_kses_post( $copyright_message ) ?>
                    </div>
                </div>
            </div>
        </div>
        <!-- End Content Protector WordPress Plugin -->
        <?php

    }

    /**
     * Replace placeholders to values in Copyright Message.
     *
     * @since 1.0.8
     * @access private
     *
     * @return string
     **/
    private function apply_replacements() {

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        /** Prepare values for Replacements. */
        $site_name = get_bloginfo( 'name' );
        $site_domain = parse_url( site_url(), PHP_URL_HOST );
        $current_date = current_time( get_option( 'date_format' ) );

        /** Do replacements. */
        $copyright_message = str_replace( [
            '{siteName}',
            '{siteDomain}',
            '{currentDate}',
        ], [
            $site_name,
            $site_domain,
            $current_date,
        ], $options['copyright_message'] );

        return wpautop( $copyright_message );

    }

    /**
     * Is page builder in admin?
     *
     * Check if we are in one of the page builders, in editor mode on the front-end.
     *
     * @since 1.0.11
     *
     * @return bool True if this is Elementor or DIVI builder, False otherwise.
     **/
    public function is_page_builder() {

        /** Elementor builder preview. */
        if ( isset( $_GET['elementor-preview'] ) ) { return true; }

        /** DIVI builder on the front-end. */
        return isset( $_GET[ 'et_fb' ] );

    }

} // End Class Sense.
