<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

use WP_Upgrader;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Class contains methods for handling AJAX requests.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class AjaxHandler {

    /**
     * Sets up a new Settings instance.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function __construct() {

        /** Is this an AJAX request? */
        if ( ! wp_doing_ajax() ) { return; }

        /** AJAX close widget. */
        add_action( 'wp_ajax__t42_content_protector__close_widget', [ $this, 'ajax_close_widget' ] );

        /** AJAX reset tab settings. */
        add_action( 'wp_ajax__t42_content_protector__reset_tab_settings', [ $this, 'ajax_reset_tab_settings' ] );

        /** AJAX subscribe. */
        add_action( 'wp_ajax__t42_content_protector__subscribe', [ $this, 'ajax_subscribe' ] );

        /** AJAX update now. */
        add_action( 'wp_ajax__t42_content_protector__update_now', [ $this, 'ajax_update_now' ] );

        /** AJAX Uninstall Now. */
        add_action( 'wp_ajax__t42_content_protector__delete_now', [ $this, 'ajax_delete_now' ] );

        /** AJAX Check Updates. */
        add_action( 'wp_ajax__t42_content_protector__check_updates', [ $this, 'ajax_check_updates' ] );

        /** AJAX Contact Support. */
        add_action( 'wp_ajax__t42_content_protector__contact_support', [ $this, 'ajax_contact_support' ] );

        /** Merlin: AJAX subscribe. */
        add_action( 'wp_ajax__t42_content_protector__merlin_subscribe', [ $this, 'ajax_merlin_subscribe' ] );

        /** Merlin: AJAX activation. */
        add_action( 'wp_ajax__t42_content_protector__merlin_activate', [ $this, 'ajax_merlin_activate' ] );

    }

    /**
     * Ajax Activate plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_merlin_activate() {

        /** Check nonce for security. */
        check_ajax_referer( 'merlin_nonce', 'wpnonce' );

        /** Prepare variables. */
        $purchase_code = filter_input( INPUT_POST, 'purchase_code', FILTER_UNSAFE_RAW );
        $is_activated = PluginActivation::get_instance()->is_activated( $purchase_code );

        /** Return activation result. */
        wp_send_json( [
            'success' => $is_activated,
        ] );

    }

    /**
     * Ajax Subscribe user.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_merlin_subscribe() {

        /** Check nonce for security. */
        check_ajax_referer( 'merlin_nonce', 'wpnonce' );

        /** Prepare variables. */
        $mail = filter_var( $_POST[ 'mail' ], FILTER_VALIDATE_EMAIL );
        $join = filter_input( INPUT_POST, 'join', FILTER_UNSAFE_RAW );
        $slug = Plugin::get_slug();

        /** Do we have valid email? */
        if ( ! $mail ) {

            wp_send_json( [
                'success' => false,
                'message' => esc_html__( 'Wrong E-mail value.', 't42-content-protector' ),
            ] );

        }

        /** Build request url. */
        $url = ( new UpdatesServer() )->get_subscribe_url( [
            'mail' => $mail,
            'join' => $join,
            'slug' => $slug,
        ] );

        /** Send subscribe request. */
        $subscribe_result = Helper::get_instance()->get_remote( $url );

        /** Return FALSE on error. */
        if ( false === $subscribe_result ) {

            wp_send_json( [
                'success' => false,
                'message' => esc_html__( 'Subscription failed.', 't42-content-protector' ),
            ] );

        }

        /** Return Subscribe result. */
        wp_send_json( [
            'success' => true,
            'message' => $subscribe_result,
        ] );

    }

    /**
     * Ajax delete plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_delete_now() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Install plugin update? */
        if ( empty( $_POST['deleteNow'] ) ) {  wp_send_json( 'Wrong Parameter Value.' ); }

        /** Check user permissions. */
        if ( ! current_user_can( 'delete_plugins' ) ) {
            wp_die( __( 'You do not have sufficient permissions to delete plugins on this site.' ) );
        }

        /** Deactivate plugin. */
        $plugin_file = Plugin::get_slug() . '/' . Plugin::get_slug() . '.php';
        if ( is_plugin_active( $plugin_file ) ) {
            deactivate_plugins( $plugin_file );
        }

        /** Check if plugin has uninstall.php file and run it. */
        $plugin_dir = WP_PLUGIN_DIR . '/' . Plugin::get_slug();
        if ( file_exists( $plugin_dir . '/uninstall.php' ) ) {
            define( 'WP_UNINSTALL_PLUGIN', true );
            include_once( $plugin_dir . '/uninstall.php' );
            unset( $GLOBALS[ 'wp_plugin_paths' ][ $plugin_dir ] );
        }

        /** Delete plugin. */
        $result = delete_plugins( [ $plugin_file ] );

        if ( is_wp_error( $result ) ) {
            $status['errorMessage'] = $result->get_error_message();
            wp_send_json_error( $status );
        } elseif ( false === $result ) {
            $status['errorMessage'] = esc_html__( 'Plugin could not be deleted.', 't42-content-protector' );
            wp_send_json_error( $status );
        }

        wp_send_json_success( esc_html__( 'Plugin deleted successfully.', 't42-content-protector' ) );

    }

    /**
     * Ajax install plugin update.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_update_now() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Install plugin update? */
        if ( empty( $_POST['updateNow'] ) ) {  wp_send_json( 'Wrong Parameter Value.' ); }

        /** Check user permissions. */
        if ( ! current_user_can( 'upload_plugins' ) ) {
            wp_die( __( 'You do not have sufficient permissions to install plugins on this site.' ) );
        }

        /** Prepare URL to update package. */
        $PluginUpdater = new PluginUpdater();
        $plugin_info = $PluginUpdater->get_plugin_information();
        if ( empty( $plugin_info->download_url ) ) {
            wp_send_json( 'Wrong Plugin Information.' );
        }
        $download_url = esc_url_raw( $plugin_info->download_url );

        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

        $options = [
            'package'                     => $download_url,
            'destination'                 => WP_PLUGIN_DIR,
            'clear_destination'           => true,
            'clear_working'               => true,
            'abort_if_destination_exists' => false,
            'hook_extra'                  => [
                'type'   => 'web',
                'action' => 'install-plugin',
            ],
        ];

        ob_start();
        $WP_Upgrader = new WP_Upgrader( new Quiet_Skin() );
        $result =  $WP_Upgrader->run( $options );
        $output = ob_get_clean();

        if ( is_wp_error( $result ) ) {
            wp_send_json( $result->get_error_message() );
        }

        /** Return result. */
        wp_send_json( [ 'result' => $result, 'output' => $output ] );

    }

    /**
     * Ajax reset tab settings.
     *
     * @since 1.0.1
     * @access public
     *
     * @return void
     **/
    public function ajax_reset_tab_settings() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Get tab slug to remove settings from. */
        $tab = filter_input( INPUT_POST, 'tab', FILTER_UNSAFE_RAW );

        /** Do we have a valid tab? */
        if ( ! Plugin::get_tabs( "$tab:enabled" ) ) {
            wp_send_json( 'Wrong Tab param.' );
        }

        /** Prepare option name where settings are stored. */
        $tab_opt = Plugin::get_snake_slug() . '_' . $tab . '_settings';

        /** Reset all settings on tab. */
        $reset_result = delete_option( $tab_opt );

        /** Return reset result. */
        wp_send_json( $reset_result );

    }

    /**
     * Ajax close widget.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_close_widget() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Prepare variables. */
        $widget_id = filter_input( INPUT_POST, 'widget_id', FILTER_UNSAFE_RAW );

        /** Do we have valid widget id? */
        if ( ! in_array( $widget_id, [ 't42-widget-help-center-btn', 't42-widget-subscribe-btn', 't42-widget-start-here-btn' ] ) ) {
            wp_send_json( 'Wrong widget id value.' );
        }

        /** Remove noise from widget id. */
        $widget_id = substr( $widget_id, 0, - 1 * strlen( '-btn' ) );
        if ( strpos( $widget_id, 't42-widget-' ) === 0 ) {
            $widget_id = substr($widget_id, strlen('t42-widget-'));
        }

        /** Prepare 'plugin_slug_widget_id_{user_id}' option name to store widget status. */
        $widget_opt = Plugin::get_snake_slug() . '_widget_' . $widget_id . '_' . get_current_user_id();

        /** Widget successfully closed for current user. */
        $closed_result = update_option( $widget_opt, '1' );

        /** Return closed result. */
        wp_send_json( $closed_result );

    }

    /**
     * Ajax subscribes to newsletters.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_subscribe() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Prepare variables. */
        $name = filter_input( INPUT_POST, 'name', FILTER_UNSAFE_RAW );
        $mail = filter_var( $_POST[ 'mail' ], FILTER_VALIDATE_EMAIL );
        $slug = Plugin::get_slug();

        /** Do we have valid email? */
        if ( ! $mail ) {
            wp_send_json( 'Wrong E-mail value.' );
        }

        /** Build request url. */
        $url = ( new UpdatesServer() )->get_subscribe_url( [
            'mail' => $mail,
            'name' => $name,
            'slug' => $slug,
        ] );

        /** Send subscribe request. */
        $subscribe_result = Helper::get_instance()->get_remote( $url );

        /** Return FALSE on error. */
        if ( false === $subscribe_result ) { wp_send_json( 'Subscription failed.' ); }

        /** Return Subscribe result. */
        wp_send_json( $subscribe_result );

    }

    /**
     * Ajax clear cache and check updates.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_check_updates() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Do we need to do a full reset? */
        if ( empty( $_POST['checkUpdates'] ) ) {  wp_send_json( 'Wrong Parameter Value.' ); }

        /** Clear cache table. */
        $cache = new Cache();
        $cache->truncate_cache_table();

        /** Render changelog to variable. */
        ob_start();
        Changelog::get_instance()->render_changelog();
        $changelog = ob_get_clean();

        /** Return "Changelog" JSON result. */
        wp_send_json( $changelog );

    }

    /**
     * Ajax contact support.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function ajax_contact_support() {

        /** Check nonce for security. */
        check_ajax_referer( 't42-content-protector', 'nonce' );

        /** Prepare variables. */
        $name = filter_input( INPUT_POST, 'name', FILTER_UNSAFE_RAW );
        $mail = filter_var( $_POST[ 'mail' ], FILTER_VALIDATE_EMAIL );
        $message = filter_input( INPUT_POST, 'message', FILTER_SANITIZE_SPECIAL_CHARS );

        $page_from = filter_input( INPUT_POST, 'page_from', FILTER_UNSAFE_RAW );
        $slug = Plugin::get_slug();

        /** Do we have valid email? */
        if ( ! $mail ) {
            wp_send_json( 'Wrong E-mail value.' );
        }

        /** Build request url. */
        $url = ( new UpdatesServer() )->get_contact_support_url();

        $data = [
            'name'         => $name,
            'mail'         => $mail,
            'message'      => $message,
            'page_from'    => $page_from,
            'slug'         => $slug,
        ];

        /** Send a message via POST request. */
        $sending_result = Helper::get_instance()->post_remote( $url, $data );

        /** Return FALSE on error. */
        if ( false === $sending_result ) { wp_send_json( 'Failed to send message.' ); }

        /** Return Subscribe result. */
        wp_send_json( $sending_result );

    }

} // End Class Settings.

