<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Base methods for AdminScripts and AdminStyles.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
 class Assets {

    /**
     * Return handle for wp_enqueue_ functions.
     *
     * @since 1.0.0
     * @access protected
     *
     * @param string $name  Name of the file. Required.
     * @param bool   $MP    Defines the folder from where to enqueue the script. Optional. Default 'false'.
     *                          false - /wp-content/plugins/{plugin-name}/js/
     *                          true - /wp-content/plugins/{plugin-name}/src/MP/assets/js/
     *
     * @return string
     **/
    public function get_handle( $name, $MP = false ) {

        /** Prepare prefix for a handle.  */
        $prefix = 't42-content-protector';

        /** Build handle. */
        $handle = $prefix . '-';

        /** Add -mp- prefix if a script is from MP core. */
        if ( $MP ) {
            $handle .= 'mp-';
        }

        $handle .= $name;

        return $handle;

    }

     /**
      * Get a relative path to script or style folder.
      *
      * @since 1.0.0
      * @access protected
      *
      * @param string $type     Folder type: css|js. Required.
      * @param bool   $MP       Defines the folder from where to enqueue the script. Optional. Default 'false'.
      *                             false - /wp-content/plugins/{plugin-name}/js|css/
      *                             true - /wp-content/plugins/{plugin-name}/src/MP/assets/js|css/
      * @param string $rel_path Relative path to the folder with css files. Optional. Default ''.
      *
      * @return string
      **/
     protected function get_folder( $type, $MP = false, $rel_path = '' ) {

         $folder = $type . '/';

         if ( $MP ) {
             $folder = 'src/MP/assets/' . $type . '/';
         }

         if ( ! empty( $rel_path ) ) {
             $folder = $rel_path;
         }

         return $folder;

     }

     /**
      * MP wrapper around wp_enqueue_script().
      *
      * @since 1.0.0
      * @access public
      *
      * @param string   $js_name     Name of the script file. Required.
      * @param bool     $MP          Defines the folder from where to enqueue the script. Optional. Default 'false'.
      *                                 false - /wp-content/plugins/{plugin-name}/js/
      *                                 true - /wp-content/plugins/{plugin-name}/src/MP/assets/js/
      * @param string[] $deps        An array of registered script handles this script depends on. Optional. Default empty array.
      * @param array|bool $args     {
      *                                 Optional.
      *                                 An array of additional script loading strategies.
      *                                 Default empty array.
      *                                 Otherwise, it may be a boolean in which case it determines whether the script is printed in the footer.
      *                                 Default false.
      *
      *      @type string    $strategy     Optional. If provided, it may be either 'defer' or 'async'.
      *      @type bool      $in_footer    Optional. Whether to print the script in the footer. Default 'true'.
      * }
      * @param string   $rel_path    Relative path to the folder with js files. Optional. Default ''.
      *
      * @return void
      **/
     public function enqueue_script( $js_name, $MP = false, $deps = [], $args = [ 'in_footer' => true ], $rel_path = '' ) {

         $this->handle_script( 'wp_enqueue_script', $js_name, $MP, $deps, $args, $rel_path );

     }

     /**
      * MP wrapper around wp_register_script().
      *
      * @since 1.0.0
      * @access public
      *
      * @param string   $js_name     Name of the script file. Required.
      * @param bool     $MP          Defines the folder from where to enqueue the script. Optional. Default 'false'.
      *                                 false - /wp-content/plugins/{plugin-name}/js/
      *                                 true - /wp-content/plugins/{plugin-name}/src/MP/assets/js/
      * @param string[] $deps        An array of registered script handles this script depends on. Optional. Default empty array.
      * @param array|bool $args     {
      *                                 Optional.
      *                                 An array of additional script loading strategies.
      *                                 Default empty array.
      *                                 Otherwise, it may be a boolean in which case it determines whether the script is printed in the footer.
      *                                 Default false.
      *
      *      @type string    $strategy     Optional. If provided, it may be either 'defer' or 'async'.
      *      @type bool      $in_footer    Optional. Whether to print the script in the footer. Default 'true'.
      * }
      * @param string   $rel_path    Relative path to the folder with js files. Optional. Default ''.
      *
      * @return void
      **/
     public function register_script( $js_name, $MP = false, $deps = [], $args = [ 'in_footer' => true ], $rel_path = '' ) {

         $this->handle_script( 'wp_register_script', $js_name, $MP, $deps, $args, $rel_path );

     }

     /**
      * MP wrapper around wp_register_script() or wp_enqueue_script().
      *
      * @since 1.0.0
      * @access public
      *
      * @param string   $wp_func     Name of the wp_register_script() or wp_enqueue_script() function. Required.
      * @param string   $js_name     Name of the script file. Required.
      * @param bool     $MP          Defines the folder from where to enqueue the script. Optional. Default 'false'.
      *                                 false - /wp-content/plugins/{plugin-name}/js/
      *                                 true - /wp-content/plugins/{plugin-name}/src/MP/assets/js/
      * @param string[] $deps        An array of registered script handles this script depends on. Optional. Default empty array.
      * @param array|bool $args     {
      *                                 Optional.
      *                                 An array of additional script loading strategies.
      *                                 Default empty array.
      *                                 Otherwise, it may be a boolean in which case it determines whether the script is printed in the footer.
      *                                 Default false.
      *
      *      @type string    $strategy     Optional. If provided, it may be either 'defer' or 'async'.
      *      @type bool      $in_footer    Optional. Whether to print the script in the footer. Default 'false'.
      * }
      * @param string   $rel_path    Relative path to the folder with js files. Optional. Default ''.
      *
      * @return void
      **/
     private function handle_script( $wp_func, $js_name, $MP = false, $deps = [], $args = [ 'in_footer' => true ], $rel_path = '' ) {
         global $wp_version;

         /** Exit, if no script. */
         if ( empty( $js_name ) ) { return; }

         /** Get a src url and full path to css file. */
         list( $src, $src_path ) = $this->get_src( 'js', $js_name, $MP, $rel_path );

         /** If >= 6.3, re-use wrapper function signature. */
         if ( version_compare( $wp_version,'6.3', '>=' ) ) {
             $wp_func(
                 $this->get_handle( $js_name, $MP ),
                 $src,
                 $deps,
                 $this->get_version( $src_path ),
                 $args
             );
         } else {

             /** Extract in_footer value for older WP version usage. */
             $in_footer = isset( $args['in_footer'] ) ? $args['in_footer'] : false;

             $wp_func(
                 $this->get_handle( $js_name, $MP ),
                 $src,
                 $deps,
                 $this->get_version( $src_path ),
                 $in_footer
             );

         }

     }

     /**
      * MP wrapper around wp_register_style().
      *
      * @since  1.0.0
      * @access public
      *
      * @param string   $css_name      Name of the css file. Required.
      * @param bool     $MP            Defines the folder from where to enqueue the script. Optional. Default 'false'. Possible values:
      *                                     false - /wp-content/plugins/{plugin-name}/css/
      *                                     true - /wp-content/plugins/{plugin-name}/src/MP/assets/css/
      * @param string[] $deps          An array of registered script handles this script depends on. Optional. Default empty array.
      * @param string   $rel_path      Relative path to the folder with css files. Optional. Default ''.
      *
      * @return void
      **/
     public function register_style( $css_name, $MP = false, $deps = [], $rel_path = '' ) {

         $this->handle_style( 'wp_register_style', $css_name, $MP, $deps, $rel_path );

     }

     /**
      * MP wrapper around wp_enqueue_style().
      *
      * @since  1.0.0
      * @access public
      *
      * @param string   $css_name      Name of the css file. Required.
      * @param bool     $MP            Defines the folder from where to enqueue the script. Optional. Default 'false'. Possible values:
      *                                     false - /wp-content/plugins/{plugin-name}/css/
      *                                     true - /wp-content/plugins/{plugin-name}/src/MP/assets/css/
      * @param string[] $deps          An array of registered script handles this script depends on. Optional. Default empty array.
      * @param string   $rel_path      Relative path to the folder with css files. Optional. Default ''.
      *
      * @return void
      **/
     public function enqueue_style( $css_name, $MP = false, $deps = [], $rel_path = '' ) {

         $this->handle_style( 'wp_enqueue_style', $css_name, $MP, $deps, $rel_path );

     }

     /**
      * MP wrapper around wp_register_style() and wp_enqueue_style().
      *
      * @since  1.0.0
      * @access public
      *
      * @param string   $wp_func            Name of the enqueue function. Required. Possible values:
      *                                     'wp_register_style' - register a style.
      *                                     'wp_enqueue_style' - enqueue a style.
      * @param string   $css_name           Name of the css file. Required.
      * @param bool     $MP                 Defines the folder from where to enqueue the script. Optional. Default 'false'. Possible values:
      *                                     false - /wp-content/plugins/{plugin-name}/css/
      *                                     true - /wp-content/plugins/{plugin-name}/src/MP/assets/css/
      * @param string[] $deps               An array of registered script handles this script depends on. Optional. Default empty array.
      * @param string   $rel_path           Relative path to the folder with css files. Optional. Default ''.
      *
      * @return void
      **/
     private function handle_style( $wp_func, $css_name, $MP = false, $deps = [], $rel_path = '' ) {

         /** Exit, if no css. */
         if ( empty( $css_name ) ) { return; }

         /** Get a src url and full path to css file. */
         list( $src, $src_path ) = $this->get_src( 'css', $css_name, $MP, $rel_path );

         /** Enqueue or Register a style. */
         $wp_func(
             $this->get_handle( $css_name, $MP ),
             $src,
             $deps,
             $this->get_version( $src_path )
         );

     }

     /**
      * Get a script version to break through caching.
      *
      * @since  1.0.0
      * @access private
      *
      * @param string $src_path Full path to script or CSS file. Required.
      *
      * @return int File modification time or current time if SCRIPT_DEBUG is true.
      **/
     private function get_version( $src_path ) {

         /**
          * The performance of filemtime() is very high.
          * On an SSD it takes 0.5 sec for 50k iterations - this is very fast!
          **/
         $version = filemtime( $src_path );

         /** If SCRIPT_DEBUG is true, use current time. */
         if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) { // Break through caching.
             $version = time();
         }

         return $version;

     }

     /**
      * MP wrapper around wp_enqueue_style().
      *
      * @since  1.0.0
      * @access public
      *
      * @param string   $file_name          Name of the js|css file. Required.
      * @param bool     $MP                 Defines the folder from where to enqueue js|css. Optional. Default 'false'. Possible values:
      *                                         false - /wp-content/plugins/{plugin-name}/css/
      *                                         true - /wp-content/plugins/{plugin-name}/src/MP/assets/css/
      * @param string   $rel_path           Relative path to the folder with css files. Optional. Default ''.
      *
      * @return array  [$src, $src_path]    Array with src url and full path to js|css file.
      **/
     private function get_src( $type, $file_name, $MP, $rel_path ) {

         /** Get a relative path to js|css folder. */
         $rel_folder = $this->get_folder( $type, $MP, $rel_path );

         /** Relative path to js|css file. */
         $rel_file = $rel_folder . $file_name . Plugin::get_suffix() . ".$type";

         /** Prepare js|css src. */
         $src      = Plugin::get_url() . $rel_file;
         $src_path = Plugin::get_path() . $rel_file;

         return [ $src, $src_path ];

     }

} // End abstract class Assets.
