<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

use stdClass;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * SINGLETON: Class contain information about the envato item.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class EnvatoItem {

    use Singleton;

    /**
     * All item details.
     *
     * @var stdClass
     * @since 1.0.0
     **/
    private static $item;

    /**
     * Sets up a new EnvatoItem instance.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function __construct() {

        /** Do we have Item details in cache? */
        $cache = new Cache( 24 * HOUR_IN_SECONDS );
        $key = Plugin::get_slug();
        $cached_item = $cache->get( $key );

        if (
            is_null( $cached_item ) || // If we haven't a cached item OR
            (
                $this->is_empty_cached_item( $cached_item ) && // We have cached item but it's empty AND
                $this->is_settings_page() // This is plugin settings page.
            )
        ) {

            /** Get item details from remote host. */
            $remote_item = $this->get_remote_item();

            /** Store item details in cache. */
            $cache->set( $key, [$key => $remote_item] );

            /** Use remote item details. */
            self::$item = $remote_item;

        }

        /** We have NOT empty cached item. */
        elseif ( ! $this->is_empty_cached_item( $cached_item ) ) {

            /** Use item details from cache. */
            $cached_item_details = json_decode( $cached_item, true );
            self::$item = $cached_item_details[$key];

        }

        /** False for all other cases. */
        else {

            self::$item = false;

        }

    }

    /**
     * Return true if item details is empty.
     *
     * @since 1.0.0
     * @access private
     *
     * @return boolean
     **/
    private function is_empty_cached_item( $cached_item ) {

        $cached_item_details = json_decode( $cached_item, true );

        $key = Plugin::get_slug();

        return empty( $cached_item_details[$key] );

    }

    /**
     * Return true if we on plugin settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return boolean
     **/
    private function is_settings_page() {

        return false !== strpos( $_SERVER[ 'REQUEST_URI' ], 'page=t42_content_protector_settings' );

    }

    /**
     * Get item details from updates server.
     *
     * @since 1.0.0
     * @access private
     *
     * @return stdClass|false
     **/
    private function get_remote_item() {

        /** Build URL to get Envato Item details by plugin slug. */
        $url = ( new UpdatesServer() )->get_plugin_details_url();

        /** Get remote plugin details. */
        $plugin_details = Helper::get_instance()->get_remote( $url );

        /** Return FALSE on error. */
        if ( false === $plugin_details ) { return false; }

        /** Convert JSON to object. */
        $plugin_details = json_decode( $plugin_details, false );

        /** Return FALSE on error. */
        if ( empty( $plugin_details ) ) { return false; }

        return $plugin_details;

    }

    /**
     * Return CodeCanyon Item URL.
     *
     * @since 1.0.0
     * @access public
     *
     * @param string $type Type of URL. Valid values: 'codecanyon-url', 'elements-url'. Default: codecanyon.
     *
     * @return string
     **/
    public function get_url( $type = 'codecanyon-url' ) {

        if ( ! is_array( self::$item ) ) { return ''; }

        /** Try to get url from plugin details. */
        if ( array_key_exists( $type, self::$item ) ) {
            return self::$item[$type];
        }

        /** Fall back for CodeCanyon URL. */
        if ( 'codecanyon-url' === $type ) {
            return 'https://1.envato.market/42themeCC';
        }

        /** Fall back for Elements URL. */
        if ( 'elements-url' === $type ) {
            return 'https://1.envato.market/WP-42Theme';
        }

        return '';

    }

    /**
     * Return Documentation URL.
     *
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public function get_documentation_url() {

        /** Try to get url from plugin details. */
        if ( is_array( self::$item ) && array_key_exists( 'documentation-url', self::$item ) ) {

            return self::$item['documentation-url'];

        }

        return '';

    }

    /**
     * Return Start Here widget content.
     *
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public function get_widget_start_here() {

        /** Try to get url from plugin details. */
        if ( is_array( self::$item ) && array_key_exists( 'widget-start-here', self::$item ) ) {

            return self::$item['widget-start-here'];

        }

        return '';

    }

    /**
     * Return CodeCanyon Item ID.
     *
     * @since 1.0.0
     * @access public
     *
     * @return int
     **/
    public function get_id() {

        /** Return stored item ID if we have it. */
        static $item_id = null;
        if ( isset( $item_id ) ) { return (int) $item_id; }

        /** Prepare 'plugin_slug_envato_id' option name to store CodeCanyon Item ID. */
        $opt_name = Plugin::get_snake_slug() . '_envato_id';

        /** Do we have stored Envato ID? */
        $item_id = get_option( $opt_name );

        /** If we have stored Envato ID, return it. */
        if ( $item_id ) { return (int) $item_id; }

        /** Else try to get id from plugin details. */
        if ( is_array( self::$item ) && array_key_exists( 'envato-item-id', self::$item ) ) {

            /** Store a local option if this is real item ID. */
            $item_id = self::$item['envato-item-id'];
            if ( $item_id > 0 ) { update_option( $opt_name, $item_id ); }

            return (int) $item_id;

        }

        /** Return 0 in all other cases. */
        return 0;

    }

} // End Class EnvatoItem.
