<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

use DOMDocument;
use DOMXPath;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Add tab "Our plugins" with other plugins from the author to the "Add Plugins" page.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class OurPlugins {

    use Singleton;

    /**
     * Sets up a new Plugin instance.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function __construct() {

        /** Add custom tab with Our plugins on the "Add Plugins" page. */
        add_filter( 'install_plugins_tabs', [ $this, 'add_our_plugins_tab' ] );

        /**
         * Add content to custom tab only if other our plugin does not use this hook.
         * We do not want to add content to the tab twice.
         **/
        if ( ! has_action( 'install_plugins_t42_plugins_tab' ) ) {
            add_action( 'install_plugins_t42_plugins_tab', [ $this, 'add_our_plugins_content' ] );
        }

    }

    /**
     * Render "Our Plugins" tab.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $tabs Current tabs: Featured | Popular | Recommended | Favorites
     *
     * @return array
     **/
    public function add_our_plugins_tab( $tabs ) {

        /** Check if the tab already exists. Created by another plugin. */
        if ( ! empty( $tabs['t42_plugins_tab'] ) ) { return $tabs; }

        /** Add "Our Plugins" tab. */
        $tabs['t42_plugins_tab'] = '
            <span>
                <img src="' . Plugin::get_url() . 'images/logo-author.svg" alt="' . esc_html__( '42Theme', 't42-content-protector' ) . '">
                <span class="label">' . esc_html__( '42Theme Plugins', 't42-content-protector' ) . '</span>
            </span>
        ';

        return $tabs;

    }

    /**
     * Add content to "Our Plugins" tab.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function add_our_plugins_content() {

        /** Render Header with description. */
        ?>
        <h2><?php esc_html__( '42Theme Plugins', 't42-content-protector' ); ?></h2>
        <p><?php esc_html__( '42Theme is a team of professional web developers. We craft unique and easy to use WordPress plugins and themes.', 't42-content-protector' ); ?></p>
        <?php

        /** Get a list of our plugins. */
        $plugins = $this->get_our_plugins();
        if ( empty( $plugins ) ) { return; }

        ?>
        <div id="t42_our_plugins_list" class="widefat">
            <?php foreach ( $plugins as $plugin ) : ?>
                <div class="plugin-card plugin-card-classic-editor">
                    <div class="plugin-card-top">
                        <div class="name column-name">
                            <h3>
                                <a href="<?php echo esc_url( $plugin->homepage ); ?>" target="_blank">
                                    <?php esc_html_e( $plugin->{'plugin-name'} ); ?>
                                    <img src="<?php esc_attr_e( $plugin->{'plugin-icon'} ); ?>" class="plugin-icon" alt="<?php esc_html_e( $plugin->{'plugin-name'} ); ?>">
                                </a>
                            </h3>
                        </div>
                        <div class="action-links">
                            <?php $this->render_actions( $plugin ); ?>
                        </div>
                        <div class="desc column-description">
                            <p><?php echo wp_kses_post( $this->create_short_description( $plugin->sections->description ) ); ?></p>
                            <p class="authors">
                                <cite>
                                    <?php esc_html_e( 'By', 't42-content-protector' ); ?>
                                    <a href="<?php echo esc_url( $plugin->author_homepage ); ?>" target="_blank">
                                        <?php esc_html_e( $plugin->author ); ?>
                                    </a>
                                </cite>
                            </p>
                        </div>
                    </div>
                    <div class="plugin-card-bottom">
                        <div class="vers column-rating">
                            <?php $this->render_rating( $plugin ); ?>
                        </div>
                        <div class="column-updated">
                            <?php $this->render_last_updated( $plugin ); ?>
                        </div>
                        <div class="column-downloaded">
                            <?php $this->render_active_installations( $plugin ); ?>
                        </div>
                        <div class="column-compatibility">
                            <?php $this->render_compatibility( $plugin ); ?>
                        </div>
                    </div>
                </div>
            <?php endforeach; ?>
        </div>
        <?php

    }

    /**
     * Render "Active Installations" block.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_active_installations( $plugin ) {

        if ( $plugin->active_installs >= 1000000 ) {
            $active_installs_millions = floor( $plugin->active_installs / 1000000 );
            $active_installs_text     = sprintf(
            /* translators: %s: Number of millions. */
                _nx( '%s+ Million', '%s+ Million', $active_installs_millions, 'Active plugin installations', 't42-content-protector' ),
                number_format_i18n( $active_installs_millions )
            );
        } elseif ( 0 === $plugin->active_installs ) {
            $active_installs_text = _x( 'Less Than 10', 'Active plugin installations', 't42-content-protector' );
        } else {
            $active_installs_text = number_format_i18n( $plugin->active_installs ) . '+';
        }

        /* translators: %s: Number of installations. */
        printf( esc_html__( '%s Active Installations', 't42-content-protector' ), $active_installs_text );

    }

    /**
     * Render "Last Updated" block.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_last_updated( $plugin ) {

        ?>
        <strong><?php esc_html_e( 'Last Updated:', 't42-content-protector' ); ?></strong>
        <?php
        $last_updated_timestamp = strtotime( $plugin->last_updated );

        /* translators: %s: Human-readable time difference. */
        printf( __( '%s ago' ), human_time_diff( $last_updated_timestamp ) );

    }

    /**
     * Render "Rating" block.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_rating( $plugin ) {

        wp_star_rating( [
            'rating' => $plugin->rating,
            'type'   => 'percent',
            'number' => $plugin->num_ratings,
        ] );
        ?>
        <span class="num-ratings" aria-hidden="true">(<?php echo number_format_i18n( $plugin->num_ratings ); ?>)</span>
        <?php

    }

    /**
     * Render "Actions" block.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_actions( $plugin ) {

        ?>
        <ul class="plugin-action-buttons">
            <li>
                <a class="button button-primary"
                   href="<?php echo esc_url( $plugin->homepage ); ?>"
                   aria-label="<?php esc_attr_e( 'Live Preview: ', 't42-content-protector' ); ?> <?php esc_attr_e( $plugin->{'plugin-name'} ); ?>"
                   target="_blank"
                >
                    <?php esc_html_e( 'Live Preview', 't42-content-protector' ); ?>
                    <span class="dashicons dashicons-external"></span>
                </a>

            </li>
            <?php $this->render_action_more_details( $plugin ); ?>
        </ul>
        <?php

    }

    /**
     * Render "More Details" link.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_action_more_details( $plugin ) {

        /** Fields to check if they have a URL. */
        $fields = [ 'codecanyon-url', 'elements-url', 'appsumo-url', 'templatemonster-url' ];

        $more_details_url = null;
        foreach ( $fields as $field ) {

            /** Skip empty fields. */
            if ( empty( $plugin->{$field} ) ) { continue; }

            /** Check if the field has a valid URL. */
            if ( filter_var( $plugin->{$field}, FILTER_VALIDATE_URL ) !== false ) {
                $more_details_url = $plugin->{$field};
                break;
            }

        }

        /** Skip if there is no URL. */
        if ( empty( $more_details_url ) ) { return; }

        ?>
        <li>
            <a href="<?php echo esc_url( $more_details_url ); ?>"
               aria-label="<?php esc_attr_e( 'More Details: ', 't42-content-protector' ); ?> <?php esc_attr_e( $plugin->{'plugin-name'} ); ?>"
               target="_blank"
            >
                <?php esc_html_e( 'More Details', 't42-content-protector' ); ?>
            </a>
        </li>
        <?php

    }

    /**
     * Render compatibility block.
     *
     * @since 1.0.0
     * @access private
     *
     * @param object $plugin - Plugin object.
     *
     * @return void
     **/
    private function render_compatibility( $plugin ) {

        $requires_wp   = isset( $plugin->requires ) ? $plugin->requires : null;
        $compatible_wp = is_wp_version_compatible( $requires_wp );
        $tested_wp     = ( empty( $plugin->tested ) || version_compare( get_bloginfo( 'version' ), $plugin->tested, '<=' ) );

        if ( ! $tested_wp ) {
            ?>
            <span class="compatibility-untested">
                <?php esc_html_e( 'Untested with your version of WordPress', 't42-content-protector' ); ?>
            </span>
            <?php
        } elseif ( ! $compatible_wp ) {
            ?>
            <span class="compatibility-incompatible">
                <strong><?php esc_html_e( 'Incompatible', 't42-content-protector' ); ?></strong>
                <?php esc_html_e( 'with your version of WordPress', 't42-content-protector' ); ?>
            </span>
            <?php
        } else {
            ?>
            <span class="compatibility-compatible">
                <strong><?php esc_html_e( 'Compatible', 't42-content-protector' ); ?></strong>
                <?php esc_html_e( 'with your version of WordPress', 't42-content-protector' ); ?>
            </span>
            <?php
        }

    }

    /**
     * Create a short description for plugin.
     *
     * @since 1.0.0
     * @access private
     *
     * @param string $description - Plugin description.
     *
     * @return string
     **/
    private function create_short_description( $description ) {

        /** Allowed tags. */
        $allowed_tags = [ 'a', 'p', 'span', 'strong', 'b', 'code', 'em', 'br', 'hr' ];

        /** Regular expression to remove all tags except allowed. */
        $pattern = '/<\s*(?!(?:' . implode( '|', $allowed_tags ) . ')\s*>)[^>]*>.*?<\/[^>]*>/i';
        $clean_description = preg_replace( $pattern, '', $description );

        /** Shorten without cutting words. */
        $p = $this->short_str( $clean_description, 400 );

        /** Remove all tags. */
        $p = strip_tags( $p );

        /** Remove trailing comma, full stop, colon, semicolon, 'a', 'A', space. */
        $p = rtrim( $p, ',.;: aA' );

        return  $p . '...';

    }

    /**
     * Create a short string, but not cut the word.
     *
     * @since 1.0.0
     * @access private
     *
     * @param string $str - String.
     * @param int $len - Length.
     * @param bool $cut - Cut string.
     *
     * @return string
     **/
    private function short_str( $str, $len, $cut = false ) {

        if ( strlen( $str ) <= $len ) { return $str; }

        return ( $cut ? substr( $str, 0, $len ) : substr( $str, 0, strrpos( substr( $str, 0, $len ), ' ' ) ) );

    }

    /**
     * Get a list of our plugins.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     **/
    private function get_our_plugins() {

        /** Key for caching. */
        $key = 't42_our_plugins';

        /** Get cached data. */
        $plugins = get_transient( $key );
        if ( false !== $plugins ) { return $plugins; }

        /** Get plugins from API. */
        $plugins = $this->get_our_plugins_from_api();
        if ( empty( $plugins ) ) { return []; }

        /** Cache data for 1 day. */
        set_transient( $key, $plugins, 7 * DAY_IN_SECONDS );

        return $plugins;

    }

    /**
     * Get a list of our plugins from updates server.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array|bool
     **/
    private function get_our_plugins_from_api() {

        /** Get "Our Plugins" API endpoint. */
        $url = ( new UpdatesServer() )->get_our_plugins_url();

        /** Get Our plugins. */
        $plugins = Helper::get_instance()->get_remote( $url );
        if ( false === $plugins ) { return false; }

        /** Convert JSON to object. */
        $plugins = json_decode( $plugins, false );
        if ( empty( $plugins ) ) { return false; }

        return $plugins;

    }

} // End class OurPlugins.
