<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Plugin class used to prepare plugin variables.
 * It can be called at any time and work without any dependencies.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class Plugin {

    use Singleton;

    const TYPE = 'plugin';

    /**
     * Plugin version.
     *
     * @since 1.0.0
     * @static
     * @access private
     * @var string
     **/
    private static $version;

    /**
     * Using minified css and js files if SCRIPT_DEBUG is turned off.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $suffix;

    /**
     * URL to plugin folder, with trailing slash.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $url;

    /**
     * Full PATH to plugin folder, with trailing slash.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $path;

    /**
     * Plugin base name.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $basename;

    /**
     * Plugin admin menu bases.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var array
     **/
    private static $menu_bases = [];

    /**
     * Full path to the main plugin file.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $plugin_file;

    /**
     * Plugin slug base.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var string
     **/
    private static $slug;

    /**
     * Settings Tabs.
     *
     * Holds the list of all tabs and fields with settings.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var array
     **/
    private static $tabs;

    /**
     * Plugin features configurations.
     *
     * @since 1.0.0
     * @static
     * @access private
     *
     * @var array
     **/
    private static $config = [];

    /**
     * Sets up a new Plugin instance.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function __construct() {

        /** Initialize main variables. */
        $this->initialization();

    }

    /**
     * Initialize main variables.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function initialization() {

        /** Full path to main plugin file. */
        self::$plugin_file = dirname( dirname( __DIR__ ) ) . '/t42-content-protector.php';

        /** Set Plugin version. */
        self::$version = self::get_plugin_data( 'Version' );

        /** Plugin slug. */
        self::$slug = self::get_plugin_data( 'TextDomain' );

        /** Gets the plugin URL (with trailing slash). */
        self::$url = plugin_dir_url( self::$plugin_file );

        /** Gets the plugin PATH. */
        self::$path = plugin_dir_path( self::$plugin_file );

        /** Using minified css and js files if SCRIPT_DEBUG is turned off. */
        self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

        /** Set plugin basename. */
        self::$basename = plugin_basename( self::$plugin_file  );

        /** Plugin settings page menu base. There may be several. */
        self::$menu_bases = $this->build_menu_bases();

        /** Fill $tabs field with default settings. */
        $this->default_tabs_settings();

        /** Fill the $config field with default settings. */
        $this->default_config_settings();

    }

    /**
     * Returns all possible menu bases for the plugin settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     **/
    private function build_menu_bases() {

        $menu_bases = [];
        $menu_bases[] = 'toplevel_page_t42_content_protector_settings';

        /** Add menu base for WPBakery plugin. */
        if ( 'wpbakery' === self::TYPE ) {
            $menu_bases[] = 'wpbakery-page-builder_page_t42_content_protector_settings';
        }

        /** Add menu base for Elementor plugin. */
        if ( 'elementor' === self::TYPE ) {
            $menu_bases[] = 'elementor_page_t42_content_protector_settings';
        }

        return $menu_bases;

    }

    /**
     * Fill $config field with default settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function default_config_settings() {

        /** Navbar right. */
        self::$config['navbar-right'] = [
            'documentation' => true, // Documentation icon in header.
            'changelog'     => true, // Changelog icon in header.
            'news'          => false, // News icon in header.
        ];

        /** Sidebar right. */
        self::$config['sidebar-right'] = [
            'enabled'       => true, // Enable/disable right sidebar.
            'start-here'    => true, // Start Here Widget.
            'help-center'   => true, // Help Center Widget.
            'subscribe'     => true, // Subscribe Widget.
        ];

        /** Merlin Wizard. */
        self::$config['merlin'] = [
            'enabled'       => true,
            'hide-external' => false
        ];

        /** Plugins list. */
        self::$config['plugins'] = [
            'rate-us'       => true, // Show "Rate Us" link.
            'rate-us-link'  => 'https://codecanyon.net/downloads', // Link to page with plugin reviews.
            'documentation' => true, // Show a link to documentation.
            'site'          => true, // Show link author site.
            'author-link'   => true, // Show the author with a link.
        ];

        /** Our Plugins. */
        self::$config['our-plugins'] = [
            'enabled'       => false, // Show/Hide "Our Plugins" tab on "Add Plugins" page.
        ];

        /** Admin menu. */
        self::$config['admin_menu'] = esc_html__( 'Content Protector', 't42-content-protector' );

    }

    /**
     * Fill $tabs field with default settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function default_tabs_settings() {

        /** Add General Tab. */
        $this->add_general_tab();

        /** Add Custom CSS Tab. */
        $this->add_custom_css_tab();

        /** Add Assignments Tab. */
        $this->add_assignments_tab();

        /** Add Activation Tab. */
        $this->add_activation_tab();

        /** Add Status Tab. */
        $this->add_status_tab();

        /** Add Updates Tab. */
        $this->add_updates_tab();

        /** Add Uninstall Tab. */
        $this->add_uninstall_tab();

        /** Add Perks Tab. */
        $this->add_perks_tab();

    }

    /**
     * Add General Tab to settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_general_tab() {

        /** If this Elementor or WPBakery plugin, do not add General Tab. */
        if ( in_array( self::TYPE, [ 'elementor', 'wpbakery' ] ) ) { return; }

        /** Create empty General Tab. */
        self::$tabs['general'] = [
            'enabled'        => true,
            'class'          => TabGeneral::class,
            'label'          => esc_html__( 'General', 't42-content-protector' ),
            'title'          => esc_html__( 'General Settings', 't42-content-protector' ),
            'show_title'     => true,
            'reset_settings' => false,
            'icon'           => 'fi-home',
            'fields'         => []
        ];

    }

    /**
     * Add Custom Css Tab to the settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_custom_css_tab() {

        /** If this Elementor or WPBakery plugin, do not add General Tab. */
        if ( in_array( self::TYPE, [ 'elementor', 'wpbakery' ] ) ) { return; }

        self::$tabs['custom_css'] = [
            'enabled'           => true,
            'class'             => TabGeneral::class,
            'label'             => esc_html__( 'Custom CSS', 't42-content-protector' ),
            'title'             => esc_html__( 'Custom CSS', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-code',
            'fields'            => [
                'custom_css' => [
                    'type'              => 'css_editor',
                    'label'             => esc_html__( 'CSS', 't42-content-protector' ),
                    'show_label'        => true,
                    'description'       => esc_html__( 'Add custom CSS here.', 't42-content-protector' ),
                    'show_description'  => true,
                    'default'           => '',
                ]
            ]
        ];

    }

    /**
     * Add the "Assignments" Tab to the settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_assignments_tab() {

        /** If this Elementor or WPBakery plugin, do not add General Tab. */
        if ( in_array( self::TYPE, [ 'elementor', 'wpbakery' ] ) ) { return; }

        self::$tabs['assignments'] = [
            'enabled'           => true,
            'class'             => TabAssignments::class,
            'label'             => esc_html__( 'Assignments', 't42-content-protector' ),
            'title'             => esc_html__( 'Assignments Settings', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-assignments',
            'assignments'       => [
                'matching_method'   => true,
                'wordpress_content' => true,
                'home_page'         => true,
                'menu_items'        => true,
                'date_time'         => true,
                'user_roles'        => true,
                'url'               => true,
                'devices'           => true,
                'custom_php'        => true,
            ],
            'fields'            => [
                'assignments' => [
                    'type'              => 'hidden',
                    'default'           => '{|matchingMethod|:1,|WPContent|:0,|WPContentVal|:||,|homePage|:0,|menuItems|:0,|menuItemsVal|:||,|dateTime|:0,|dateTimeStart|:||,|dateTimeEnd|:||,|languages|:0,|languagesVal|:||,|userRoles|:0,|userRolesVal|:||,|URL|:0,|URLVal|:||,|devices|:0,|devicesVal|:||,|os|:0,|osVal|:||,|browsers|:0,|browsersVal|:||,|mobileBrowsersVal|:||,|IPs|:0,|IPsVal|:||,|PHP|:0,|PHPVal|:||}',
                ]
            ]
        ];

    }

    /**
     * Add Activation Tab to settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_activation_tab() {

        $item_id = EnvatoItem::get_instance()->get_id();

        /** Not show if plugin doesn't have Envato ID. */
        if ( ! $item_id ) { return; }

        /** Special case for non-envato plugins without purchase code, activation and with ID: 1. */
        if ( 1 === $item_id ) { return; }

        self::$tabs['activation'] = [
            'enabled'           => true,
            'class'             => TabActivation::class,
            'label'             => esc_html__( 'Activation', 't42-content-protector' ),
            'title'             => esc_html__( 'Plugin Activation', 't42-content-protector' ),
            'show_title'        => false,
            'reset_settings'    => false,
            'icon'              => 'fi-key',
            'fields'            => [
                'envato_purchase_code_' . $item_id => [
                    'type'              => 'text',
                    'default'           => '',
                ]
            ]
        ];

    }

    /**
     * Add Status Tab to settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_status_tab() {

        self::$tabs['status'] = [
            'enabled'           => true,
            'class'             => TabStatus::class,
            'label'             => esc_html__( 'Status', 't42-content-protector' ),
            'title'             => esc_html__( 'System Status', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-status',
            'reports'           => [
                'server'    => [
                    'enabled'               => true,
                    'os'                    => true,
                    'software'              => true,
                    'mysql_version'         => true,
                    'php_version'           => true,
                    'write_permissions'     => true,
                    'zip_installed'         => true,
                    'curl_installed'        => true,
                    'allow_url_fopen'       => false,
                    'elementor_installed'   => false,
                    'dom_installed'         => false,
                    'xml_installed'         => false,
                    'bcmath_installed'      => false,
                    'mbstring_installed'    => false,
                    'server_time'           => false,
                ],
                'wordpress'     => [
                    'enabled' => true
                ],
                'plugins'       => [
                    'enabled' => true
                ],
                'theme'         => [
                    'enabled' => true
                ],
            ]
        ];

        /** Check if WPBakery is installed for WPBakery plugin. */
        if ( 'wpbakery' === self::TYPE ) {
            self::$tabs['status'][ 'reports' ][ 'server' ][ 'wpbakery_installed' ]  = true;
        }

        /** Check if Elementor is installed for Elementor plugin. */
        if ( 'elementor' === self::TYPE ) {
            self::$tabs['status'][ 'reports' ][ 'server' ][ 'elementor_installed' ]  = true;
        }

    }

    /**
     * Add the "Updates" Tab to the settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_updates_tab() {

        self::$tabs['updates'] = [
            'enabled'           => true,
            'class'             => TabGeneral::class,
            'label'             => esc_html__( 'Updates', 't42-content-protector' ),
            'title'             => esc_html__( 'Updates', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-updates',
            'fields'            => [
                'check_updates' => [
                    'type'              => 'button',
                    'label'             => esc_html__( 'Check for updates', 't42-content-protector' ),
                    'show_label'        => false,
                    'button_text'       => esc_html__( 'Check Updates', 't42-content-protector' ),
                    'description'       => '',
                    'show_description'  => false,
                    'icon'              => 'refresh',
                    'attr'              => [
                        'class'     => 't42-button-primary',
                    ]
                ],
                'changelog'     => [
                    'type'              => 'changelog',
                    'render'            => [ Changelog::get_instance(), 'render_changelog' ],
                    'label'             => esc_html__( 'Changelog', 't42-content-protector' ),
                    'show_label'        => false,
                    'description'       => '',
                    'show_description'  => false
                ]
            ]
        ];

        /** Add "New Version Available" notice. */
        $this->add_new_version_notice_field();

    }

    /**
     * Add "New Version Available" notice field to the "Updates" Tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_new_version_notice_field() {

        $PluginUpdater = new PluginUpdater();
        $plugin_info = $PluginUpdater->get_plugin_information();
        if ( empty( $plugin_info->version ) ) { return; }

        $latest_version = $plugin_info->version;
        $current_version = self::get_version();

        /** Show an update notice if the current version is less than the latest version. */
        if ( ! ( version_compare( $latest_version, $current_version ) > 0 ) ) { return; }

        $fields = self::$tabs['updates']['fields'];

        /** Index of 'check_updates' field, after which we will insert 'update notice' field. */
        $insert_index = array_search( 'check_updates', array_keys( $fields ), true );
        if ( $insert_index === false ) { return; }

        /** Add 'update notice' field. */
        self::$tabs['updates']['fields'] = array_merge(
            array_slice( $fields, 0, $insert_index + 1, true ),
            [
                'update_notice' => [
                    'type'    => 'html',
                    'content' => $this->get_update_notice_content( $latest_version )
                ]
            ],
            array_slice( $fields, $insert_index + 1, null, true )
        );

    }

    /**
     * Return "New Version Available" notice content.
     *
     * @since 1.0.0
     * @access private
     *
     * @param string $latest_version - Latest plugin version.
     *
     * @return string
     **/
    private function get_update_notice_content( $latest_version ) {

        ob_start();

        ?>
        <div class="t42-update-warning-box t42-alert-warning t42-alert" t42-alert="">
            <span t42-icon="icon: fi-alert; ratio: 1.3"></span>
            <p>
                <?php esc_html_e( 'There is a new version of', 't42-content-protector' ); ?>
                <?php echo self::get_name(); ?>
                <?php esc_html_e( 'available.', 't42-content-protector' ); ?>

                <a href="<?php echo esc_url(
                    network_admin_url(
                        'plugin-install.php?tab=plugin-information&plugin=t42-content-protector&section=changelog&TB_iframe=true&width=772&height=726'
                    )
                ) ?>" class="thickbox open-plugin-details-modal">
                    <?php esc_html_e( 'View version', 't42-content-protector' ); ?>
                    <?php esc_html_e( $latest_version ); ?>
                    <?php esc_html_e( 'details', 't42-content-protector' ); ?></a><?php

                /** Message for activated users. */
                if ( PluginActivation::get_instance()->is_activated() ) {

                    require_once ABSPATH . 'wp-includes/pluggable.php';
                    /** @noinspection SpellCheckingInspection */
                    echo sprintf( esc_html__( ' or %supdate now%s', 't42-content-protector' ),
                        '<button id="t42_content_protector_updates_settings-update_now" 
                                        class="t42-button t42-button-default t42-waves-effect t42-waves-light"
                                ><span class="t42-spinner-border"></span>',
                        '</button>'
                    );

                } else {

                    ?><br><?php
                    esc_html_e( 'Automatic update is unavailable for unactivated plugin.', 't42-content-protector' );
                    ?><br><?php
                    $envato_item = EnvatoItem::get_instance();
                    /** @noinspection SpellCheckingInspection */
                    echo sprintf(
                        esc_html__( 'Please %sactivate your copy%4$s to enable automatic updates or visit the plugin page 
                on %sCodeCanyon%4$s or %sEnvato Elements%4$s to download the latest version.', 't42-content-protector' ),
                        '<a href="' . admin_url( 'admin.php?page=t42_content_protector_settings&tab=activation' ) . '" class="t42-link">',
                        '<a href="' . esc_url( $envato_item->get_url() ) . '" target="_blank" class="t42-link">',
                        '<a href="' . esc_url( $envato_item->get_url( 'elements-url' ) ) . '" target="_blank" class="t42-link">',
                        '</a>'
                    );

                }
                ?>

            </p>
        </div>
        <?php

        return ob_get_clean();

    }

    /**
     * Add Uninstall Tab to settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_uninstall_tab() {

        self::$tabs['uninstall'] = [
            'enabled'           => true,
            'class'             => TabGeneral::class,
            'label'             => esc_html__( 'Uninstall', 't42-content-protector' ),
            'title'             => esc_html__( 'Uninstall Settings', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-uninstall',
            'fields'            => [
                'delete_plugin' => [
                    'type'              => 'select',
                    'label'             => esc_html__( 'Delete mode', 't42-content-protector' ),
                    'show_label'        => true,
                    'description'       => esc_html__( 'Choose which data to delete on uninstall action.', 't42-content-protector' ),
                    'show_description'  => true,
                    'options'           => [
                        'plugin'                => esc_html__( 'Delete Plugin only', 't42-content-protector' ),
                        'plugin+settings'       => esc_html__( 'Delete Plugin and Settings', 't42-content-protector' ),
                        'plugin+settings+data'  => esc_html__( 'Delete Plugin, Settings and Data', 't42-content-protector' ),
                    ],
                    'default'           => ['plugin'],
                ],
                'divider_delete_now'  => [ 'type' => Controls::DIVIDER ],
                'delete_now'          => [
                    'type'              => 'button',
                    'label'             => esc_html__( 'Uninstall Plugin', 't42-content-protector' ),
                    'show_label'        => false,
                    'button_text'       => esc_html__( 'Uninstall Now', 't42-content-protector' ),
                    'description'       => '',
                    'show_description'  => false,
                    'icon'              => 'trash',
                    'attr'              => [
                        'class'     => 't42-button-danger',
                        'id'        =>'t42-delete-now-btn'
                    ]
                ],
            ]
        ];

    }

    /**
     * Add Perks Tab to settings page.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function add_perks_tab() {

        self::$tabs['perks'] = [
            'enabled'           => true,
            'class'             => TabGeneral::class,
            'label'             => esc_html__( 'Perks', 't42-content-protector' ),
            'title'             => esc_html__( 'Check current Perks', 't42-content-protector' ),
            'show_title'        => true,
            'reset_settings'    => false,
            'icon'              => 'fi-star',
            'fields'            => [
                'beget' => [
                    'type'    => 'html',
                    'content' => $this->get_perk(
                        'https://beget.com/p141492/en',
                        'https://static.42theme.com/42/perks/beget-logo.png',
                        esc_html__( 'Beget', 't42-content-protector' ),
                        "<p>Virtual and Premium web hosting. Dedicated servers.</p>
                                <p>Evaluation period of 30 days will allow you to assure yourself of our service quality, of the efficiency, qualification and friendliness of our technical support team and to opt for us.</p>"
                    ),
                ],

                'elementor' => [
                    'type'    => 'html',
                    'content' =>$this->get_perk(
                        'https://elementor.com?ref=3174',
                        'https://static.42theme.com/42/perks/elementor-logo-bordeaux-full.png',
                        esc_html__( 'Elementor', 't42-content-protector' ),
                        "<p>Elementor: #1 Free WordPress Website Builder.</p>
                                <p>Build and customize every part of your website intuitively with Elementor’s visual builder. Zero coding required.</p>"
                    ),
                ],

                'elements' => [
                    'type'    => 'html',
                    'content' =>$this->get_perk(
                        'https://1.envato.market/all-items',
                        'https://static.42theme.com/42/perks/envato-elements-logo.png',
                        esc_html__( 'Envato Elements', 't42-content-protector' ),
                        "<p>Unlimited downloads of 56+ million creative assets. From just $16.50/month.</p>
                                <p>Elements opens up a world of creative possibilities for all your design projects. Powered by a community of talented designers, our subscription puts great design in reach for everyone.</p>"
                    ),
                ],

                'wpkraken' => [
                    'type'    => 'html',
                    'content' =>$this->get_perk(
                        'https://wpkraken.io/?ref=drinkmaker',
                        'https://static.42theme.com/42/perks/wpkraken-logo-dark.png',
                        esc_html__( 'WP Kraken', 't42-content-protector' ),
                        "<p>WordPress Experts At Your Service.</p>
                                <p>Meet an in-house team of web development & design experts. We offer website care, on-demand projects and dedicated staff.</p>"
                    ),
                ],

                'codeable' => [
                    'type'    => 'html',
                    'content' =>$this->get_perk(
                        'https://codeable.io/?ref=99TAr',
                        'https://static.42theme.com/42/perks/codeable-logo.svg',
                        esc_html__( 'Codeable', 't42-content-protector' ),
                        "<p>Hire the best WordPress developers. Vetted by us, perfect for you!</p>
                                <p>Free estimate. No obligation to hire. 100% risk-free.</p>"
                    ),
                ],

                //


            ]
        ];

    }

    /**
     * Render Perk card.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    private function get_perk( $link = '#', $logo_url = '', $title = '', $content = '' ) {

        return <<<CONTENT
<div class="t42-perk-box">
    <a href="$link" target="_blank" rel="noopener noreferrer">
        <div class="t42-perks-logo"
             style="background-image: url('$logo_url');"></div>
    </a>
    <h4 class="t42-perks-title">$title</h4>
    $content
    <div class="t42-perks-footer">
        <a href="$link" 
           target="_blank" 
           class="t42-button t42-button-primary t42-waves-effect t42-waves-light"
           rel="noopener noreferrer"
        >    
            <span class="t42-text">Get Perk</span>
        </a>
    </div>
</div>
CONTENT;

    }

    /**
     * Return current plugin metadata.
     *
     * @param string $field - Field name from plugin data.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string|array {
     *     Plugin data. Values will be empty if not supplied by the plugin.
     *
     *     @type string $Name        Name of the plugin. Should be unique.
     *     @type string $Title       Title of the plugin and link to the plugin's site (if set).
     *     @type string $Description Plugin description.
     *     @type string $Author      Author's name.
     *     @type string $AuthorURI   Author's website address (if set).
     *     @type string $Version     Plugin version.
     *     @type string $TextDomain  Plugin textdomain.
     *     @type string $DomainPath  Plugins relative directory path to .mo files.
     *     @type bool   $Network     Whether the plugin can only be activated network-wide.
     *     @type string $RequiresWP  Minimum required version of WordPress.
     *     @type string $RequiresPHP Minimum required version of PHP.
     * }
     **/
    private static function get_plugin_data( $field ) {

        static $plugin_data;

        /** We already have $plugin_data. */
        if ( $plugin_data !== null ) {
            return $plugin_data[$field];
        }

        /** This is first time call of method, so prepare $plugin_data. */
        if ( ! function_exists('get_plugin_data') ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }

        $plugin_data = get_plugin_data( self::get_plugin_file() );

        return $plugin_data[$field];

    }

    /**
     * Get Plugin version.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_version() {
        return self::$version;
    }

    /**
     * Get Plugin Name.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_name() {
        return self::get_plugin_data( 'Name' );
    }

    /**
     * Get .min suffix.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_suffix() {
        self::create_instance();
        return self::$suffix;
    }

    /**
     * Get URL to plugin folder with trailing slash.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_url() {
        self::create_instance();
        return self::$url;
    }

    /**
     * Get full Path to plugin folder with trailing slash.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_path() {
        self::create_instance();
        return self::$path;
    }

    /**
     * Get Plugin Basename.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_basename() {
        self::create_instance();
        return self::$basename;
    }

    /**
     * Get plugin menu bases.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return array
     **/
    public static function get_menu_bases() {
        self::create_instance();
        return self::$menu_bases;
    }

    /**
     * Get a path to plugin file.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_plugin_file() {

        return  dirname( dirname( __DIR__ ) ) . '/t42-content-protector.php';

    }

    /**
     * Get Plugin slug.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_slug() {

        return self::get_plugin_data( 'TextDomain' );

    }

    /**
     * Get Plugin slug in snake case.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return string
     **/
    public static function get_snake_slug() {

        return Helper::kebab_to_snake( self::get_plugin_data( 'TextDomain' ) );

    }

    /**
     * Get Plugin tabs or tabs value by key.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return array|string|null
     **/
    public static function get_tabs( $key = null ) {

        self::create_instance();

        /** Return full tabs if key is empty. */
        if ( null === $key ) {
            return self::$tabs;
        }

        /** Return element or null if not found. */
        return self::search_by_key( $key, self::$tabs );

    }

    /**
     * Get Plugin config value or full array.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @param string|null $key Access key in format: 'name:sub-name:element'
     *
     * @return array|string|null
     **/
    public static function get_config( $key = null ) {

        self::create_instance();

        /** Return full config if key is empty. */
        if ( null === $key ) {
            return self::$config;
        }

        /** Return element or null if not found. */
        return self::search_by_key( $key, self::$config );

    }

    /**
     * Return element or null if not found.
     *
     * @param string $key Access key in format: 'name:sub-name:element'
     * @param array $arr Array with settings to search
     *
     * @since 1.0.0
     * @access public
     *
     * @return array|string|null
     **/
    private static function search_by_key( $key, $arr ) {

        /** Convert sting to array. */
        $keys = explode( ':', $key );

        /** Search element by $keys array. */
        $temp =& $arr;
        foreach ( $keys as $i ) {
            $temp =& $temp[$i];
        }

        /** Return element or null if not found. */
        return $temp;

    }

    /**
     * Set Plugin tabs.
     *
     * @param array $tabs - Tabs and fields with settings.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public static function set_tabs( $tabs ) {

        self::$tabs = $tabs;

    }

    /**
     * Set Plugin features configuration.
     *
     * @param array $config - Features configuration.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public static function set_config( $config ) {

        self::$config = $config;

    }

} // End class Plugin.
