<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/**
 * SINGLETON: Class used to implement Activation tab on plugin settings page.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 */
final class PluginActivation {

    use Singleton;

    /**
     * Sets up a new PluginActivation instance.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function __construct() {

        /** Work only in the admin area. */
        if ( ! is_admin() ) { return; }

        $item_id = EnvatoItem::get_instance()->get_id();
        add_action( 'update_option_envato_purchase_code_' . $item_id, [$this, 'reset_temporary_activation'] );

    }

    /**
     * Reset temporary activation on every pid change.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function reset_temporary_activation() {

        delete_transient ( 'activated_' . EnvatoItem::get_instance()->get_id() );

        /** Reset updates cache. */
        PluginUpdater::reset_cache();

    }

    /**
     * Get purchase code for activation.
     *
     * @since 1.0.0
     * @access private
     *
     * @param false $pid
     *
     * @return string|false
     **/
    private function get_purchase_code( $pid = false ) {

        if ( $pid !== false ) { return $pid; }

        /** Get fresh PID from form. */
        $envato_pic = 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id();

        /** Special case for non-envato plugins without purchase code. */
        if ( 'envato_purchase_code_1' === $envato_pic ) {
            return strrev( base64_decode( 'MjRkMjRiYTliNmY1LTI0YWItZWYyNC15ZWsxLWFyZW5lZzI0' ) );
        }

        if ( isset( $_POST['t42_content_protector_activation_settings'][$envato_pic] ) ) {
            $purchase_code = filter_input( INPUT_POST, $envato_pic );
        }

        /** Or get PID from an option. */
        else {
            $purchase_code = get_option( 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id() );
        }

        return $purchase_code;

    }

    /**
     * Return Activation Status.
     *
     * @since 1.0.0
     * @access public
     *
     * @param bool|string $pid - Purchase Code. Optional.
     *
     * @return boolean True if activated.
     **/
    public function is_activated( $pid = false ) {

        /** Get purchase code for activation. */
        $purchase_code = $this->get_purchase_code( $pid );

        /** If we do not have $purchase_code then nothing to check. */
        if ( ! $purchase_code ) {  return false; }

        /** Clean and validate purchase code. */
        /** Remove spaces. */
        $purchase_code = trim( $purchase_code );

        /** Make sure the code is valid before sending it to Envato. */
        if ( ! preg_match( "/^(\w{8})-((\w{4})-){3}(\w{12})$/", $purchase_code ) ) { return false; }

        /** Check temporary activation */
        $local_activation = $this->local_validation(); // 0 - if no local value, go to download.

        if ( $local_activation === 0 ) {

            /** Need Remote validation. */
            if ( $this->remote_validation( $purchase_code ) ) {

                $this->temporary_activation( true, $purchase_code );

                /** Reset updates cache. */
                PluginUpdater::reset_cache();

                return true;

            }

            /** Not activated. */
            $this->temporary_activation( false, $purchase_code );

            return false;

        }

        /** Use local activation. */
        $this->temporary_activation( $local_activation, $purchase_code );

        return filter_var( $local_activation, FILTER_VALIDATE_BOOLEAN );

    }

    /**
     * Set temporary activation.
     *
     * @since  1.0.0
     * @access private
     *
     * @param bool $activate - Temporary Activate/Deactivate.
     * @param string $purchase_code - Envato Purchase Code. Optional.
     *
     * @return void
     **/
    private function temporary_activation( $activate, $purchase_code = '' ) {

        /** Timeout for temporary activation. */
        $hours = 48;

        if ( filter_var( $activate, FILTER_VALIDATE_BOOLEAN ) ) {

            set_transient( 'activated_' . EnvatoItem::get_instance()->get_id(), '1', $hours * HOUR_IN_SECONDS );

            /** Write purchase code to option. */
            $key = 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id();
            update_option( 't42_content_protector_activation_settings', [$key => $purchase_code] );

            /**
             * The Purchase Code entered by the user must be stored in the wp_options table as a string
             * using the following syntax:
             *    option_name: envato_purchase_code_<item_id>
             *    option_value: <purchase_code>
             **/
            update_option( $key, $purchase_code );

        } else {

            set_transient( 'activated_' . EnvatoItem::get_instance()->get_id(), '0', $hours * HOUR_IN_SECONDS );

        }

    }

    /**
     * Validate PID on local server.
     *
     * @return int|true|false - status of temporary activation status.
     *         0 - if we haven't a local activation status and need remove validation.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function local_validation() {

        /** Get temporary activation status. */
        $tmp_activation_status = get_transient( 'activated_' . EnvatoItem::get_instance()->get_id() );

        /** Send a query to server if we haven't temporary option. */
        if ( false === $tmp_activation_status ) {
            return 0;
        }

        return filter_var( $tmp_activation_status, FILTER_VALIDATE_BOOLEAN );

    }

    /**
     * Validate PID on our server.
     *
     * @param $purchase_code - Envato Purchase Code.
     *
     * @since 1.0.0
     * @access private
     *
     * @return boolean
     **/
    private function remote_validation( $purchase_code ) {

        /** Build request url. */
        $url = ( new UpdatesServer() )->get_actions_url( [
            'action'  => 'validate',
            'plugin'  => 't42-content-protector',
            'domain'  => $this->get_domain(),
            'version' => Plugin::get_version(),
            'pid'     => $purchase_code,
            'admin_e' => base64_encode( get_option( 'admin_email' ) ),
        ] );

        /** Download JSON file with purchase code validation status true/false. */
        $json = wp_remote_get( $url, [
            'timeout' => 15,
            'headers' => [
                'Accept' => 'application/json'
            ]
        ] );

        /** We’ll check whether the answer is correct. */
        if ( is_wp_error( $json ) ) { return false; }

        /** Have answered with the wrong code. */
        if ( wp_remote_retrieve_response_code( $json ) !== 200 ) { return false; }

        return true === json_decode( $json['body'], true );

    }

    /**
     * Return current domain. Parsed from site url.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    private function get_domain() {

        return parse_url( site_url(), PHP_URL_HOST );

    }

} // End Class PluginActivation.
