<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

use DateTime;
use Exception;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * SINGLETON: Class used to implement base plugin features.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class PluginHelper {

    use Singleton;

    /**
     * Sets up a new PluginHelper instance.
     *
     * @since 1.0.0
     * @access public
     **/
    private function __construct() {

        /** Add plugin links. */
        add_filter( 'plugin_action_links_' . Plugin::get_basename(), [$this, 'add_links'] );

        /** Add plugin meta. */
        add_filter( 'plugin_row_meta', [$this, 'add_row_meta'], 10, 3 );

        /** Load JS and CSS for Backend Area. */
        $this->enqueue_backend();

        /** Add custom tab with "Our plugins" on the "Add Plugins" page. */
        if ( Plugin::get_config( 'our-plugins:enabled' ) ) {
            OurPlugins::get_instance();
        }

    }

    /**
     * Load JS and CSS for Backend Area.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function enqueue_backend() {

        /** Add admin styles. */
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ] );

        /** Add admin javascript. */
        add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );

    }

    /**
     * Add CSS for admin area.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function admin_styles() {

        /** Get current screen or exit. */
        $screen = get_current_screen();
        if ( null === $screen ) { return; }

        /** Add styles on WP Plugins page. */
        if ( 'plugins' === $screen->base ) {

            /** Enqueue a plugins.css. */
            AdminStyles::get_instance()->enqueue_style( 'plugins', true );

        }

        /** Add styles on WP Add new Plugins page. */
        if ( 'plugin-install' === $screen->base ) {

            /** Enqueue a plugin-install-plugins-tab.css. */
            AdminStyles::get_instance()->enqueue_style( 'plugin-install-plugins-tab', true );

        }



    }

    /**
     * Add JS for admin area.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function admin_scripts() {

        /** Get current screen or exit. */
        $screen = get_current_screen();
        if ( null === $screen ) { return; }

        /** Add scripts only on WP Plugins page. */
        if ( 'plugins' !== $screen->base ) { return; }

        /** Enqueue a plugins.js. */
        $AS = AdminScripts::get_instance();
        $AS->enqueue_script( 'plugins', true );

        /** Prepare values to pass to JS. */
        $to_js = [
            'pluginsURL' => plugins_url(),
        ];
        wp_localize_script(
            $AS->get_handle( 'plugins', true ),
            't42ContentProtector',
            $to_js
        );

    }

    /**
     * Add additional links on plugin page.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $links Current links: Deactivate | Edit
     *
     * @return array
     **/
    public function add_links( $links ) {

        /** Add 'Settings' link to beginning. */
        array_unshift( $links, '<a title="' . esc_html__( 'Settings', 't42-content-protector' ) . '" href="' . admin_url( 'admin.php?page=t42_content_protector_settings' ) . '">' . esc_html__( 'Settings', 't42-content-protector' ) . '</a>' );

        /** Add 'Author' link to ending. */
        if ( Plugin::get_config( 'plugins:site' ) ) {
            $links[] = '<a href="https://42theme.com/" target="_blank" class="t42-author-site"><img src="' . Plugin::get_url() . 'images/logo-author.svg" alt="' . esc_html__( '42Theme', 't42-content-protector' ) . '"> 42Theme</a>';
        }

        return $links;

    }

    /**
     * Add "Rate us", "Documentation" and other links on the Plugins page.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array  $plugin_meta The array having default links: Version 2.0.0 | By {Author} | Visit plugin site
     * @param string $file The name of the plugin file.
     * @param array  $plugin_data {
     *      An array of plugin data.
     *
     *          @type string   $id Plugin ID, e.g. `w.org/plugins/[plugin-name]`.
     *          @type string   $slug Plugin slug.
     *          @type string   $plugin Plugin basename.
     *          @type string   $new_version New plugin version.
     *          @type string   $url Plugin URL.
     *          @type string   $package Plugin update package URL.
     *          @type string[] $icons An array of plugin icon URLs.
     *          @type string[] $banners An array of plugin banner URLs.
     *          @type string[] $banners_rtl An array of plugin RTL banner URLs.
     *          @type string   $requires The version of WordPress which the plugin requires.
     *          @type string   $tested The version of WordPress the plugin is tested against.
     *          @type string   $requires_php The version of PHP which the plugin requires.
     *          @type string   $upgrade_notice The upgrade notice for the new plugin version.
     *          @type bool     $update -supported Whether the plugin supports updates.
     *          @type string   $Name The human-readable name of the plugin.
     *          @type string   $PluginURI Plugin URI.
     *          @type string   $Version Plugin version.
     *          @type string   $Description Plugin description.
     *          @type string   $Author Plugin author.
     *          @type string   $AuthorURI Plugin author URI.
     *          @type string   $TextDomain Plugin textdomain.
     *          @type string   $DomainPath Relative path to the plugin's .mo file(s).
     *          @type bool     $Network Whether the plugin can only be activated network-wide.
     *          @type string   $RequiresWP The version of WordPress which the plugin requires.
     *          @type string   $RequiresPHP The version of PHP which the plugin requires.
     *          @type string   $UpdateURI ID of the plugin for update purposes, should be a URI.
     *          @type string   $Title The human-readable title of the plugin.
     *          @type string   $AuthorName Plugin author's name.
     *          @type bool     $update Whether there's an available update. Default null.
     *  }
     *
     * @return array
     **/
    public function add_row_meta( $plugin_meta, $file, $plugin_data ) {

        /** Exit if not our plugin. */
        if ( Plugin::get_basename() !== $file ) { return $plugin_meta; }

        /** Build plugin meta-links from scratch. */
        $plugin_meta = [];

        /** Plugin Version. */
        $plugin_meta = $this->add_plugin_version( $plugin_meta, $plugin_data );

        /** Plugin Author. */
        $plugin_meta = $this->add_plugin_author( $plugin_meta, $plugin_data );

        /** View details popup. */
        $plugin_meta = $this->add_plugin_view_details( $plugin_meta, $plugin_data );

        /** Visit plugin site. */
        $plugin_meta = $this->add_plugin_site( $plugin_meta, $plugin_data );

        /** Add 'Documentation' link. */
        $plugin_meta = $this->add_plugin_documentation( $plugin_meta );

        /** Add "Rate Us" link. */
        return $this->add_plugin_rate_us( $plugin_meta );

    }

    /**
     * Add a plugin version to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     * @param array $plugin_data An array of plugin data.
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_version( $plugin_meta, $plugin_data ) {

        /** Plugin Version. */
        if ( ! empty( $plugin_data['Version'] ) ) {
            /* translators: %s: Plugin version number. */
            $plugin_meta[] = sprintf( esc_html__( 'Version %s', 't42-content-protector' ), $plugin_data['Version'] );
        }

        return $plugin_meta;

    }

    /**
     * Add a plugin author to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     * @param array $plugin_data An array of plugin data.
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_author( $plugin_meta, $plugin_data ) {

        /** Plugin Author. */
        if ( ! empty( $plugin_data['Author'] ) ) {
            $author = $plugin_data['Author'];

            /** Add a link to the Author. */
            if ( ( ! empty( $plugin_data['AuthorURI'] ) && Plugin::get_config( 'plugins:author-link' ) ) ) {
                $author = '<a href="' . $plugin_data['AuthorURI'] . '" target="_blank">' . $plugin_data['Author'] . '</a>';
            }

            /* translators: %s: Plugin author name. */
            $plugin_meta[] = sprintf( esc_html__( 'By %s', 't42-content-protector' ), $author );
        }

        return $plugin_meta;

    }

    /**
     * Add a plugin view details popup to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     * @param array $plugin_data An array of plugin data.
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_view_details( $plugin_meta, $plugin_data ) {

        /** View details popup. */
        if ( current_user_can( 'install_plugins' ) ) {
            $plugin_meta[] = sprintf(
                '<a href="%s" class="thickbox open-plugin-details-modal" aria-label="%s" data-title="%s">%s</a>',
                esc_url(
                    network_admin_url(
                        'plugin-install.php?tab=plugin-information&plugin=t42-content-protector&TB_iframe=true&width=600&height=550'
                    )
                ),
                /* translators: %s: Plugin name. */
                esc_attr( sprintf( esc_html__( 'More information about %s', 't42-content-protector' ), $plugin_data['Name'] ) ),
                esc_attr( $plugin_data['Name'] ),
                esc_html__( 'View details', 't42-content-protector' )
            );
        }

        return $plugin_meta;

    }

    /**
     * Add a plugin site to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     * @param array $plugin_data An array of plugin data.
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_site( $plugin_meta, $plugin_data ) {

        /** Visit plugin site. */
        if ( ! empty( $plugin_data['PluginURI'] ) ) {
            /* translators: %s: Plugin name. */
            $aria_label = sprintf( esc_html__( 'Visit plugin site for %s', 't42-content-protector' ), $plugin_data['Name'] );

            $plugin_meta[] = sprintf(
                '<a href="%s" aria-label="%s" target="_blank">%s</a>',
                esc_url( $plugin_data['PluginURI'] ),
                esc_attr( $aria_label ),
                esc_html__( 'Visit plugin site', 't42-content-protector' )
            );
        }

        return $plugin_meta;

    }

    /**
     * Add a documentation link to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_documentation( $plugin_meta ) {

        /** Add 'Documentation' link to the beginning. */
        if ( Plugin::get_config( 'plugins:documentation' ) ) {
            $plugin_meta[] = '<a title="' . esc_html__( 'Documentation', 't42-content-protector' ) . '" href="' . esc_url( EnvatoItem::get_instance()->get_documentation_url() ) . '" target="_blank">' .
                             esc_html__( 'Documentation', 't42-content-protector' ) .
                             '</a>';
        }

        return $plugin_meta;

    }

    /**
     * Add a link to "Rate Us" to the meta-row.
     *
     * @since  1.0.0
     * @access public
     *
     * @param array $plugin_meta The array having plugin links: Version 2.0.0 | By {Author} | Visit plugin site
     *
     * @return array $plugin_meta
     **/
    private function add_plugin_rate_us( $plugin_meta ) {

        /** Exit if "Rate Us" is disabled in config. */
        if ( ! Plugin::get_config( 'plugins:rate-us' ) ) { return $plugin_meta; }

        /** Get special class for special days. */
        $special_class = $this->get_special_class();

        /** Get "Rate Us" URL. */
        $rate_us_url = Plugin::get_config( 'plugins:rate-us-link' );
        if ( empty( $rate_us_url ) ) { return $plugin_meta; }

        $plugin_meta[] = esc_html__( 'Rate us:', 't42-content-protector' ) . "
        <a href='" . esc_url( $rate_us_url ) . "' 
           target='_blank' 
           rel='noopener' 
           class='t42-rating-link'
           title='" . esc_html__( 'Please take one minute to leave a review about your experience with us', 't42-content-protector' ) . "'>
            <span class='t42-rating-stars " . esc_attr__( $special_class ) . "'>
                <span></span><span></span><span></span><span></span><span></span>
            </span>
        </a>
        ";

        return $plugin_meta;

    }

    /**
     * Get special class for special days.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    private function get_special_class() {

        /** St. Valentine's Day. */
        if ( $this->is_date_between( '02-13', '02-15' ) ) {
            return 't42-st-valentines-day';
        }

        /** New Year. */
        if (
            $this->is_date_between( '12-30', '12-31' ) ||
            $this->is_date_between( '01-01', '01-02' )
        ) {
            return 't42-new-year-day';
        }

        /** Women's Day. */
        if ( $this->is_date_between( '03-07', '03-07' ) ) {
            return 't42-womens-day';
        }

        /** Fool's Day. */
        if ( $this->is_date_between( '04-01', '04-01' ) ) {
            return 't42-fools-day';
        }

        /** Webmaster's day. */
        if ( $this->is_date_between( '04-04', '04-04' ) ) {
            return 't42-webmasters-day';
        }

        /** Astronautics day. */
        if ( $this->is_date_between( '04-12', '04-12' ) ) {
            return 't42-astronautics-day';
        }

        /** First day of Winter. */
        if ( $this->is_date_between( '12-1', '12-3' ) ) {
            return 't42-winter-first-day';
        }

        /** First day of Spring. */
        if ( $this->is_date_between( '03-1', '03-3' ) ) {
            return 't42-spring-first-day';
        }

        /** First day of Summer. */
        if ( $this->is_date_between( '06-1', '06-3' ) ) {
            return 't42-summer-first-day';
        }

        /** First day of Autumn. */
        if ( $this->is_date_between( '09-1', '09-3' ) ) {
            return 't42-autumn-first-day';
        }

        return '';

    }

    /**
     * Return true if the current day in an interval.
     *
     * @since 1.0.0
     * @access private
     *
     * @param string $start - Month and day numbers in format 'mm-dd' for interval start.
     * @param string $end - Month and day numbers in format 'mm-dd' for an interval end.
     *
     * @return boolean
     **/
    private function is_date_between( $start, $end ) {

        try {

            /** Get the current date. */
            $now = new DateTime();
            $now->setTime(0, 0);
            $year = date( 'Y' );

            $start_date = new DateTime( $year . '-' . $start );
            $end_date = new DateTime( $year . '-' . $end );

            return $start_date <= $now && $now <= $end_date;

        } catch ( Exception $e ) {

            return false;

        }

    }

} // End Class PluginHelper.

