<?php
/**
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use stdClass;

/**
 * Class used to implement plugin update mechanism.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
class PluginUpdater {

    /**
     * In this option, we temporary store plugin info.
     *
     * @since 1.0.0
     * @access private
     **/
    const OPT_PLUGIN_INFO = 't42_content_protector_plugin_info';

    /**
     * Sets up a new PluginUpdater instance.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Work only in the admin area. */
        if ( ! is_admin() ) { return; }

        /** Disable if plugin don't have Envato ID. */
        $plugin_id = EnvatoItem::get_instance()->get_id();
        if ( $plugin_id === 0 ) { return; }

        /** Check do we have a new version? */
        add_filter( 'site_transient_update_plugins', [$this, 'update_plugins'] );
        add_filter( 'transient_update_plugins', [$this, 'update_plugins'] );

        /** Show a custom update message in a plugin list. */
        add_action( 'in_plugin_update_message-t42-content-protector/t42-content-protector.php', [$this, 'show_plugin_update_message' ], 10, 2 );

        /** Add plugin info to View details popup.  */
        add_filter( 'plugins_api', [$this, 'plugin_info'], 10, 3 );

    }

    /**
     * Check do we have a new version?
     *
     * @since 1.0.0
     * @access public
     *
     * @param $update_plugins
     *
     * @return mixed
     **/
    public function update_plugins( $update_plugins ) {

        /** Quick sanity checks. */
        if ( ! is_object( $update_plugins ) ) { return $update_plugins; }

        /** 'response' must be an array. */
        if ( ! isset( $update_plugins->response ) || ! is_array( $update_plugins->response ) ) {
            $update_plugins->response = [];
        }

        /** Don't do anything if there's already a response for this plugin. */
        if ( ! empty( $update_plugins->response['t42-content-protector/t42-content-protector.php'] ) ) {
            return $update_plugins;
        }

        /** Reset temporary cache. */
        $this->force_check();

        /**
         * Get plugin information from cache or remote.
         * This method runs multiple times, so we use short time cache.
         **/
        $plugin_info = $this->get_plugin_information();

        /** Can't get plugin information. */
        if (
            ( false === $plugin_info ) ||
            ( null === $plugin_info ) || // Wrong JSON.
            ( empty( $plugin_info->version ) ) // When Envato return: "Error: Failed to get data for this item."
        ) { return $update_plugins; }

        /** Show plugin update if the latest version is newer. */
        return $this->compare_with_new( $plugin_info, $update_plugins );

    }

    /**
     * Show plugin update if the latest version is newer.
     *
     * @param $plugin_info - Remote Plugin information.
     * @param $update_plugins - Current $update_plugins object.
     *
     * @since 1.0.0
     * @access public
     *
     * @return object
     **/
    private function compare_with_new( $plugin_info, $update_plugins ) {
        global $wp_version;

        $current_version = Plugin::get_version();
        $latest_version = $plugin_info->version;

        /** If the Latest version is newer, show an update version. */
        if ( version_compare( $latest_version, $current_version ) > 0 ) {

            $update_plugins->response['t42-content-protector/t42-content-protector.php'] = (object)[
                'slug'        => 't42-content-protector',
                'plugin'      => 't42-content-protector/t42-content-protector.php',
                'new_version' => $latest_version, // The newest version
                'package'     => ( ! empty( $plugin_info->download_url ) ) ? $plugin_info->download_url : '',
                'tested'      => $wp_version,
                'icons' => [
                    '2x' => Plugin::get_url() . 'images/logo-color.svg',
                    '1x' => Plugin::get_url() . 'images/logo-color.svg',
                ]
            ];

        }

        return $update_plugins;

    }

    /**
     * Get plugin information from cache or remote.
     *
     * @since 1.0.0
     * @access public
     *
     * @return false|object
     **/
    public function get_plugin_information() {

        /**
         * Method is called foreach installed plugin,
         * so we go to the cache table only once per request.
         **/
        static $plugin_info = null;
        if ( null !== $plugin_info ) {
            return $plugin_info;
        }

        /** Trying to get from cache first. */
        $cache = new Cache();
        $cache_key = self::OPT_PLUGIN_INFO;
        $plugin_info = $cache->get( $cache_key );
        if ( ! empty( $plugin_info ) ) {
            $plugin_info = json_decode( $plugin_info, true );
            $plugin_info = (object) reset( $plugin_info );
            return $plugin_info;
        }

        /** Add item_id and pid params to url before request. */
        $url = ( new UpdatesServer() )->get_updates_url();

        /** Download plugin JSON file with the actual plugin information on our server if the cache is empty. */
        $response = wp_remote_get( $url, [
            'timeout' => 10,
            'headers' => [
                'Accept' => 'application/json'
            ]
        ] );

        /** We’ll check whether the answer is correct. */
        if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) !== 200 ) {
            $cache->set( $cache_key, [ $cache_key => false ] );
            return false;
        }

        $plugin_info = json_decode( wp_remote_retrieve_body( $response ), false );

        /** Handle empty response and cache the result. */
        if ( empty( $plugin_info ) ) {
            $cache->set( $cache_key, [ $cache_key => false ] );
            return false;
        }

        /** Cache the plugin information. */
        $cache->set( $cache_key, [ $cache_key => $plugin_info ] );

        return $plugin_info;

    }

    /**
     * Prepare plugin info.
     *
     * @since 1.0.0
     * @access public
     *
     * @param false|object|array $result The result object or array. Default false.
     * @param string             $action The type of information being requested from the Plugin Installation API.
     * @param object             $args   Plugin API arguments.
     *
     * @return bool|stdClass
     **/
    public function plugin_info( $result, $action, $args ) {

        /** Reset temporary cache. */
        $this->force_check();

        /** Do nothing if this is not about getting plugin information. */
        if ( $action !== 'plugin_information' || 't42-content-protector' !== $args->slug ) {
            return $result;
        }

        /** Trying to get from cache first. */
        $remote = $this->get_plugin_information();
        if ( empty( $remote->version ) ) { return $result; }

        return $this->prepare_plugin_data( $remote );

    }

    /**
     * Prepare plugin data for View details popup.
     *
     * @since 1.0.0
     * @access private
     *
     * @param stdClass $remote
     *
     * @return stdClass
     **/
    private function prepare_plugin_data( $remote ) {
        global $wp_version;

        $result = new stdClass();

        $result->name            = $remote->name;
        $result->slug            = $remote->slug;
        $result->version         = $remote->version;
        $result->last_updated    = $remote->last_updated;
        $result->active_installs = $remote->active_installs;

        /** Rating. */
        if ( isset( $remote->rating ) ) {
            $result->rating      = $remote->rating;
            $result->num_ratings = $remote->num_ratings;
        }

        $result->tested       = $wp_version; // Tested up to WordPress version.
        $result->requires     = $remote->requires; // Requires an at least WordPress version.
        $result->requires_php = $remote->requires_php; // The minimum required version of PHP.
        $result->homepage     = $remote->homepage;
        $result->author       = '<a href="' . esc_url( $remote->author_homepage ) . '" target="_blank">' . esc_html( $remote->author ) . '</a>';

        /** Prepare contributors. */
        if ( isset( $remote->contributors ) && is_array( $remote->contributors ) ) {
            $result->contributors = array_map(
                static function( $contributor ) {
                    return (array) $contributor;
                },
                $remote->contributors
            );
        }

        /** Download url returned only for valid PID.  */
        if ( isset( $remote->download_url ) ) {
            $result->download_link = $remote->download_url;
            $result->trunk = $remote->download_url;
        }

        // TODO: Can we rewrite code to use array?
        //       $remote->sections is always array?
        /** Fix sections. Array to object. */
        if ( is_array( $remote->sections ) ) {
            $remote->sections = (object) $remote->sections;
        }

        $result->sections = [
            'description'  => $remote->sections->description,
            'installation' => $remote->sections->installation,
            'changelog'    => $remote->sections->changelog,
        ];

        /** FAQ section. */
        if ( isset( $remote->sections->faq ) ) {
            $result->sections['faq'] = $remote->sections->faq;
        }

        /** Reviews section. */
        if ( isset( $remote->sections->reviews ) ) {
            $result->sections['reviews'] = $remote->sections->reviews;
        }

        /** Screenshots section. */
        if ( isset( $remote->sections->screenshots ) ) {
            $result->sections['screenshots'] = $remote->sections->screenshots;
        }

        /** Banners. */
        if ( isset( $remote->banners ) ) {
            $result->banners = (array) $remote->banners;
        }

        return $result;

    }

    /**
     * Reset temporary options on Force update check.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function force_check() {

        /** Run only once per request. */
        static $done = false;
        if ( $done ) { return; }
        $done = true;

        /** If user pressed "Check again" Link. */
        if ( isset( $_GET['force-check'] ) && $_GET['force-check'] === '1' ) {
            self::reset_cache();
        }

    }

    /**
     * Reset plugin update cache.
     *
     * @since 1.0.0
     * @access public
     * @static
     *
     * @return void
     **/
    public static function reset_cache() {

        delete_transient( 't42_content_protector_upgrade' );
        delete_transient( self::OPT_PLUGIN_INFO );

        /** Delete Cache. */
        $cache = new Cache();
        $cache->delete( self::OPT_PLUGIN_INFO );

        /** Key for caching "Our Plugins". */
        delete_transient('t42_our_plugins');

    }

    /**
     * Show custom update messages on plugins page.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function show_plugin_update_message() {

        /** Message for non-activated users. */
        if ( ! PluginActivation::get_instance()->is_activated() ) {

            ?><br /><span class="t42-line">&nbsp;</span><?php

            $envato_item = EnvatoItem::get_instance();
            /** @noinspection SpellCheckingInspection */
            echo sprintf(
                esc_html__( 'Please %sactivate your copy%4$s to enable automatic updates or visit the plugin page 
                on %sCodeCanyon%4$s or %sEnvato Elements%4$s to download the latest version.', 't42-content-protector' ),
                '<a href="' . admin_url( 'admin.php?page=t42_content_protector_settings&tab=activation' ) . '">',
                '<a href="' . esc_url( $envato_item->get_url() ) . '" target="_blank">',
                '<a href="' . esc_url( $envato_item->get_url( 'elements-url' ) ) . '" target="_blank">',
                '</a>'
            );

        }

    }

} // End Class PluginUpdater.
