<?php
/**
 * Content Protector for WordPress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 *
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2016-2023 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ContentProtector;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * Return API endpoints.
 *
 * @since 1.0.0
 * @author Alexander Khmelnitskiy
 **/
final class UpdatesServer {

    /**
     * URL to updates server.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    const UPDATES_URL = 'https://updates.42theme.com/';

    /**
     * Return API endpoint for "Our Plugins" tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    public function get_our_plugins_url() {

        /** Build URL to get "Our Plugins" tab content. */
        $url = self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/get-our-plugins';
        $url .= '?plugin-slug=' . urlencode( Plugin::get_slug() );

        return $url;

    }

    /**
     * Return API endpoint for Changelog file.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    public function get_changelog_url() {

        /** Build changelog url. */
        return self::UPDATES_URL . 'changelogs/' . Plugin::get_slug() . '.html';

    }

    /**
     * Return API endpoint for a Subscribe form.
     *
     * @since 1.0.0
     * @access private
     *
     * @param array $data Data to pass to the endpoint.
     *
     * @return string
     **/
    public function get_subscribe_url( $data = [] ) {

        /** Build request url. */
        $url = self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/subscribe?';

        /** Add parameters if they are passed. */
        foreach ( $data as $key => $value ) {
            $url .= $key . '=' . $value . '&';
        }

        return $url;

    }

    /**
     * Return API endpoint for updates.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    public function get_updates_url() {

        /** Check if there's a new version of the plugin. */
        $url = self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/updates?';
        $url .= 'plugin-slug=' . Plugin::get_slug();

        /** If this copy is activated. */
        if ( PluginActivation::get_instance()->is_activated() ) {

            /** Add PID to url. */
            $purchase_code = get_option( 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id() );
            $url .= '&pid=' . $purchase_code;

        }

        return $url;

    }

    /**
     * Return API endpoint for a contact support form.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    public function get_contact_support_url() {

        /** Build request url. */
        return self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/contact-support';

    }

    /**
     * Return API endpoint for plugin details.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     **/
    public function get_plugin_details_url() {

        /** Build URL to get Envato Item details by plugin slug. */
        $url = self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/get-plugin-details';
        $url .= '?plugin-slug=' . urlencode( Plugin::get_slug() );

        return $url;

    }

    /**
     * Return API endpoint for Actions.
     *
     * @since 1.0.0
     * @access private
     *
     * @param array $data Data to pass to the endpoint.
     *
     * @return string
     **/
    public function get_actions_url( $data = [] ) {

        /** Build request url. */
        $url = self::UPDATES_URL . 'wp-json/t42-purchase-validator/v1/actions?';

        /** Add parameters if they are passed. */
        foreach ( $data as $key => $value ) {
            $url .= $key . '=' . $value . '&';
        }

        return $url;

    }

} // End Class UpdatesServer.
